/********************************************************************************
 * Created on Jul 6, 2005
 *
 * Copyright (c) 2005 University of Southampton
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE.
 *******************************************************************************/

package org.pasoa.common;

import java.util.Map;
import org.pasoa.storage.StorageSystem;
import org.w3c.dom.Document;


/**
 * A PlugIn process a DOM Document using the StorageSystem and returns an 
 * appropriate Document either containing a result document or an error message.
 * Exceptions should only be thrown when fatal. A PlugIn should make every effort
 * to handle problems and return appropriate error messages.
 * 
 * PlugIns are expected to be thread safe. i.e. a plug in should be expect to
 * be called from multiple threads.
 * 
 * @author Paul Groth pg03r@ecs.soton.ac.uk
 */
public interface PlugIn
{
    public Map getConfiguration ();
    
    /**
     * At least the following values are present in all plug-in configurations:
     * <UL>
     *    <LI> The provenance service URL, with key org.pasoa.common.Constants.PROVENANCE_STORE_URL
     * </UL>
     */
    public void initialize (Map configuration);
    public Document process (Document document, StorageSystem ss) throws Exception;
    
    public boolean receiveDispatchesSyncronously();
    
    /**
     * This method should only be used when the PlugIn receives dispatches asynchrounsly.
     * @return
     */
    public String getDefaultResponse();
    
}
