package org.pasoa.common.structure;

import java.io.Serializable;

/**
 * Represents a global key that accesses a p-assertion.
 * 
 * @author pgroth, sm
 */
public class GlobalPAssertionKey implements Serializable
{
    private InteractionKey record;
    private String view;
    private String local;

    /**
     * Construct a GlobalPAssertionKey, which points to a particular p-assertion
     * uniquely
     * 
     * @param interactionRecordID the interaction record where the p-assertion
     *            resides
     * @param viewName the view in the interaction record where the p-assertion
     *            resides
     * @param localPAssertionID the local id of the p-assertion which
     *            distinguishes it from all other p-assertions in that view
     */
    public GlobalPAssertionKey(InteractionKey interactionRecordID,
            String viewName, String localPAssertionID)
    {

        record = interactionRecordID;
        view = viewName;
        local = localPAssertionID;
    }

    /**
     * Checks whether one GlobalPAssertionKey is equal to another. Links are
     * ignored during the comparison.
     */
    public boolean equals(Object other)
    {
        return other instanceof GlobalPAssertionKey
               && ((GlobalPAssertionKey) other).record.equals (record)
               && ((GlobalPAssertionKey) other).view.equals (view)
               && ((GlobalPAssertionKey) other).local.equals (local);
    }

    public String getLocalPAssertionID()
    {
        return local;
    }

    public InteractionKey getInteractionKey()
    {
        return record;
    }

    public String getViewName()
    {
        return view;
    }

    public int hashCode()
    {
        return record.hashCode () + view.hashCode () + local.hashCode ();
    }

}
