package org.pasoa.common.structure;

import java.io.Serializable;
import org.pasoa.dommap.DOMWriter;
import org.w3c.dom.Element;

/**
 * An interaction key uniquely identifies an interaction and therefore an
 * interaction record. An interaction is the exchange of a message from a
 * sending actor (the client) to a receiving actor (the service).
 * 
 * @author pgroth
 * 
 */
public class InteractionKey implements Serializable
{
    private Element source;
    private Element sink;
    private String interaction;

    /**
     * Construct an interaction key.
     * 
     * @param messageSource identifies the source of the interaction, i.e. where
     *            the message is sent from
     * @param messageSink identifes the sink of the interaction, i.e. where the
     *            message is received at
     * @param interactionID an id that distinguishes this interaction from all
     *            other interactions between the given messageSource and
     *            messsageSink
     */
    public InteractionKey(Element messageSource, Element messageSink,
            String interactionID)
    {
        source = messageSource;
        sink = messageSink;
        interaction = interactionID;
    }

    public boolean equals(Object other)
    {
        return (other instanceof InteractionKey)
               && ((InteractionKey) other).source.isEqualNode (source)
               && ((InteractionKey) other).sink.isEqualNode (sink)
               && ((InteractionKey) other).interaction.equals (interaction);
    }

    public String getInteractionID()
    {
        return interaction;
    }

    public Element getMessageSink()
    {
        return sink;
    }

    public Element getMessageSource()
    {
        return source;
    }

    public int hashCode()
    {
        return interaction.hashCode ();
    }

    public String toString()
    {
        StringBuffer sb = new StringBuffer ();
        sb.append ("source: " + DOMWriter.writeToString (sink));
        sb.append (" sink: " + DOMWriter.writeToString (source));
        sb.append (" id: " + interaction);
        return sb.toString ();
    }
}
