package org.pasoa.common.structure;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

/**
 * Represents a record of an interaction.
 * @author pgroth, sm
 *
 */
public class InteractionRecord implements Serializable {
    private InteractionKey id;
    private View                client;
    private View                service;
    private List                inferred;

    /**
     * Construct an InteractionRecord
     * 
     * @param recordID the interaction key uniquely identifying this record
     * @param senderView the sender's (client) view of the interaction
     * @param receiverView the recievier's (service) view of the interaction
     */
    public InteractionRecord (InteractionKey recordID, View senderView, View receiverView) {
        id       = recordID;
        client   = senderView;
        service  = receiverView;
        inferred = new LinkedList ();
    }
    
    /**
     * Used for the q-structure: includes inferred views.
     */
    public InteractionRecord (InteractionKey recordID, View senderView, View receiverView, List inferredViews) {
        id       = recordID;
        client   = senderView;
        service  = receiverView;
        inferred = inferredViews;
    }
    
    /**
     * A client view may return null, if none is present.
     * @return
     */
    public View getSenderView () {
        return client;
    }
    
    public List getInferredViews () {
        return inferred;
    }
    
    public InteractionKey getInteractionKey () {
        return id;
    }
    
    /**
     * A service view may return null if none is present
     * 
     */
    public View getReceiverView () {
        return service;
    }
    
    public void setSenderView (View view) {
        client = view;
    }
    
    public void setReceiverView (View view) {
        service = view;     
    }
}
