package org.pasoa.common.structure;

import org.w3c.dom.Element;

/**
 * Represents an object id in a relationship p-assertion. An object id
 * is a pointer to a p-assertion and possibly part of its contents.
 * The object id also identifies the parameter the data is associated with in the
 * relationship. If the p-assertion is located in another provenance store
 * an link may also be included that points to the store where p-assertion is.
 *
 * @author pgroth, sm
 *
 */
public class ObjectID extends PAssertionDataKey {
    private String _paramName;
    private Link   _link;
    
    public ObjectID (GlobalPAssertionKey gpak, String parameterName, Element dataAccessor, Link objectLink) {
        super (gpak, dataAccessor);
        _paramName = parameterName;
        _link      = objectLink;
    }
    
    public ObjectID (PAssertionDataKey data, String parameterName, Link objectLink) {
        this (data.getGlobalPAssertionKey (), parameterName, data.getDataAccessor (), objectLink);
    }
    
    public boolean equals (Object otherPADK) {
        ObjectID other;
        
        if (otherPADK instanceof ObjectID) {
            other = ((ObjectID) otherPADK);
            return super.equals (other) && other.getParameterName ().equals (_paramName) && other.getLink ().equals (_link);
        } else {
            return false;
        }
    }
    
    public Link getLink () {
        return _link;
    }
    
    public String getParameterName () {
        return _paramName;
    }
    
    public Element getObjectDataAccessor () {
        return super.getDataAccessor ();
    }
    
    public int hashCode () {
        int hash = super.hashCode ();
        
        if (_paramName != null) {
            hash += _paramName.hashCode ();
        }
        if (_link != null) {
            hash += _link.hashCode ();
        }
        
        return hash;
    }
}
