package org.pasoa.common.structure;

import org.pasoa.common.structure.InteractionKey;
import org.pasoa.common.structure.PAssertion;
import org.w3c.dom.Element;

/**
 * Represents a P-Assertion to be recorded in a provenance store.
 * @author pgroth, sm
 */
public class Record 
{
    private PAssertion          assertion;
    private InteractionKey      ids;
    private String              view;
    private Element             theAsserter;
    
    /**
     * A Record defines a P-Assertion to be recorded, 
     * what interaction record and view the P-Assertion should be recorded
     * under in the provenance store and finally the asserter of that message.
     * 
     * @see RecordSf for recording the submission finished message
     * 
     * @param passertion the p-assertion to be recorded
     * @param identifiers identifies the interaction that the p-assertion should be
     *                      recorded under
     * @param viewKind this should be either Constants.CLIENT_VIEW or Constants.SERVICE_VIEW
     *                   and identifies the view in the interaction that the p-assertion 
     *                   should be recorded under.
     * @param asserter  the asserter of the p-assertion
     */
    public Record (PAssertion passertion, 
                   InteractionKey identifiers, 
                   String viewKind, 
                   Element asserter) {
        
        assertion = passertion;
        ids       = identifiers;
        view  = viewKind;
        theAsserter = asserter;
    }

    public String getViewKind () {
        return view;
    }
    
    public InteractionKey getIdentifiers () {
        return ids;
    }
    
    public PAssertion getPAssertion () {
        return assertion;
    }
    
    public Element getAsserter()
    {
        return theAsserter;
    }

}
