package org.pasoa.common.structure;

import java.util.List;
import org.w3c.dom.Element;

/**
 * A Relationship p-assertion. Represents a relationship between a p-assertion
 * or its parts with other p-assertions (and their parts). 
 * @author pgroth
 *
 */
public class Relationship extends PAssertion
{
    private SubjectID subject;
    private String relate;
    private List objects;

    /**
     * Construct a relationship between p-assertions.
     * Relationships are one to many tuples consisting of 
     * a subject, relationship, and many objects
     * 
     * @param localPAssertionID the id of this relationship p-assertion
     * @param subjectLocalPAssertionID 
     * @param subjectDataAccessor if there is no subject data accessor pass null
     * @param subjectParameterName
     * @param relation
     * @param objectIDs a list of object ids
     */
    public Relationship(String localPAssertionID,
                        String subjectLocalPAssertionID, 
                        Element subjectDataAccessor,
                        String subjectParameterName, 
                        String relation, 
                        List objectIDs)
    {
        super (localPAssertionID);
        subject = new SubjectID (subjectLocalPAssertionID, 
                                 subjectDataAccessor, 
                                 subjectParameterName);
        relate = relation;
        objects = objectIDs;
    }

    public Relationship (String localPAssertionID, SubjectID subjectID, String relation, List objectIDs) {
        super (localPAssertionID);
        subject = subjectID;
        relate  = relation;
        objects = objectIDs;
    }
    
    public List getObjectIDs()
    {
        return objects;
    }

    public String getRelation()
    {
        return relate;
    }

    public SubjectID getSubject()
    {
        return subject;
    }
}
