package org.pasoa.common.structure;

import java.io.Serializable;
import java.util.List;

import org.w3c.dom.Element;

/**
 * Represents a View of an interaction by an actor.
 * 
 * @author pgroth
 * 
 */
public class View implements Serializable
{
    private List interactions;
    private List states;
    private List relations;
    private int numExpectedMsgs;
    private Link link;
    private Element theAsserter;

    /**
     * Construct a View
     * 
     * @param interactionPAssertions a list of interaction p-assertions
     * @param actorStatePAssertions a list of actor state p-assertions
     * @param relationships a list relationship p-assertions
     * @param numberOfExpectedPAssertions the number of expected p-assertions
     *            for this view
     * @param asserter the asserter of the assertions for the view
     * @param viewLink a reference to the provenance store that contains the
     *            opposite view of the interaction (optional)
     */
    public View(List interactionPAssertions, List actorStatePAssertions,
            List relationships, int numberOfExpectedPAssertions,
            Element asserter, Link viewLink)
    {
        interactions = interactionPAssertions;
        states = actorStatePAssertions;
        relations = relationships;
        numExpectedMsgs = numberOfExpectedPAssertions;
        link = viewLink;
        theAsserter = asserter;
    }

    public View(List interactionPAssertions, List actorStatePAssertions,
            List relationships, int numberOfExpectedMsgs, Element asserter)
    {
        this (interactionPAssertions, actorStatePAssertions, relationships, numberOfExpectedMsgs, asserter, null);
    }

    public List getActorStatePAssertions()
    {
        return states;
    }

    public List getInteractionPAssertions()
    {
        return interactions;
    }

    public List getRelationships()
    {
        return relations;
    }

    /**
     * If the method returns -1, the number of expected assertions has not yet been
     * set.
     * 
     * @return
     */
    public int getNumberOfExpectedPAssertions()
    {
        return numExpectedMsgs;
    }

    public Element getAsserter()
    {
        return theAsserter;
    }

    public Link getViewLink()
    {
        return link;
    }

    public void setNumberOfExpectedPAssertions(int total)
    {
        numExpectedMsgs = total;
    }
}
