package org.pasoa.common.structure.deserialiser;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.pasoa.common.Constants;
import org.pasoa.common.structure.ActorStatePAssertion;
import org.pasoa.common.structure.GlobalPAssertionKey;
import org.pasoa.common.structure.InteractionContext;
import org.pasoa.common.structure.Link;
import org.pasoa.common.structure.InteractionPAssertion;
import org.pasoa.common.structure.InteractionKey;
import org.pasoa.common.structure.ObjectID;
import org.pasoa.common.structure.PAssertionDataKey;
import org.pasoa.common.structure.PHeader;
import org.pasoa.common.structure.Relationship;
import org.pasoa.dommap.DOMWriter;
import org.pasoa.dommap.SimpleDocument;
import org.pasoa.dommap.SimpleElement;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Provides convience methods for deserialising p-structure XML 
 * into the classes defined in org.pasoa.common.structure
 * @author sm, pgroth
 *
 */
public class PStructureDeserialiser {
    public static Element getFirstChildElement (Element parent) {
        NodeList children = parent.getChildNodes ();
        Node     child;
        
        for (int index = 0; index < children.getLength (); index += 1) {
            child = children.item (index);
            if (child instanceof Element) {
                return (Element) child;
            }
        }
        
        return null;
    }
    
    public ActorStatePAssertion getActorStatePAssertion (Element serialised) {
        String localPAssertionID = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "localPAssertionId").item (0).getChildNodes ().item (0).getNodeValue ();
        
        return new ActorStatePAssertion (localPAssertionID, serialised);
    }
    
    public GlobalPAssertionKey getGlobalPAssertionID (Element serialised) {
        InteractionKey interactionRecordID = getInteractionRecordID (serialised);
        String         viewName            = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "viewKind").item (0).getChildNodes ().item (0).getNodeValue ();
        String         localPAssertionID   = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "localPAssertionId").item (0).getChildNodes ().item (0).getNodeValue ();
        
        return new GlobalPAssertionKey (interactionRecordID, viewName, localPAssertionID);
    }
    
    public InteractionContext getInteractionContext (Element serialised) {
        InteractionKey interactionKey      = getInteractionRecordID ((Element) serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "interactionKey").item (0).getChildNodes ().item (0));
        List           interactionMetadata = new LinkedList ();
        NodeList       metadata            = serialised.getElementsByTagNameNS (Constants.PHEADER_NS, "interactionMetaData");
        
        for (int index = 0; index <= metadata.getLength (); index += 1) {
            interactionMetadata.add (metadata.item (index));
        }
        
        return new InteractionContext (interactionKey, interactionMetadata);
    }
    
    public InteractionKey getInteractionRecordID (Element serialised) {
        NodeList found;
        String   interactionId;
        Node     sink;
        Node     source;
        
        if (serialised == null) {
            throw new RuntimeException ("PStructureDeserialiser.getInteractionRecordID: serialised interaction key element is null");
        }
        found         = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "interactionId");
        if (found.getLength () > 0) {
            interactionId = found.item (0).getChildNodes ().item (0).getNodeValue ();
        } else {
            throw new RuntimeException ("PStructureDeserialiser.getInteractionRecordID: serialised interaction key has no ps:interactionId child");
        }
        sink   = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "messageSink").  item (0);
        source = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "messageSource").item (0);
        
        return new InteractionKey (getFirstChildElement ((Element) source), getFirstChildElement ((Element) sink), interactionId);
    }
    
    public InteractionPAssertion getInteractionPAssertion (Element serialised) {
        String localPAssertionID = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "localPAssertionId").item (0).getChildNodes ().item (0).getNodeValue ();
        String transformation    = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "documentationStyle").item (0).getChildNodes ().item (0).getNodeValue ();
        
        return new InteractionPAssertion (localPAssertionID, transformation, serialised);
    }
    /*
    public PAssertionDataKey getPAssertionDataKey (Element serialised) {
        Element gpakNode     = (Element) serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "globalPAssertionKey").item (0);
        Node    accessorNode = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "dataAccessor").item (0);
        
        return new PAssertionDataKey (getGlobalPAssertionID (gpakNode), (Element) accessorNode.getChildNodes ().item (0));
    }*/
    
    public PHeader getPHeader (Element serialised) {
        List           interactionMetadata = new LinkedList ();
        List           interactionContexts = new LinkedList ();
        NodeList       metadata            = serialised.getElementsByTagNameNS (Constants.PHEADER_NS, "interactionMetaData");
        NodeList       contexts            = serialised.getElementsByTagNameNS (Constants.PHEADER_NS, "interactionContext");
        NodeList       found;
        InteractionKey interactionKey;

        found = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "interactionKey");
        if (found.getLength () > 0) {
            interactionKey = getInteractionRecordID ((Element) found.item (0));
        } else {
            throw new RuntimeException ("PStructureDeserialiser.getPHeader: p-header has no ps:interactionKey element, serialised p-header element has name " + serialised.getNodeName ());
        }
        for (int index = 0; index < metadata.getLength (); index += 1) {
            interactionMetadata.add (metadata.item (index));
        }
        for (int index = 0; index < contexts.getLength (); index += 1) {
            interactionContexts.add (getInteractionContext ((Element) contexts.item (index)));
        }
        
        return new PHeader (interactionKey, interactionMetadata, interactionContexts);
    }
    
    public Relationship getRelationshipAssertion (Element serialised) {
        Element  subjectIDNode            = (Element) serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "subjectId").item (0);
        NodeList objectIDNodes            = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "objectId");
        String   localPAssertionID        = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "localPAssertionId").item (0).getChildNodes ().item (0).getNodeValue ();
        String   subjectLocalPAssertionID = subjectIDNode.getElementsByTagNameNS (Constants.PSTRUCT_NS, "localPAssertionId").item (0).getChildNodes ().item (0).getNodeValue ();
        String   subjectParameterName     = subjectIDNode.getElementsByTagNameNS (Constants.PSTRUCT_NS, "parameterName").item (0).getChildNodes ().item (0).getNodeValue ();
        String   relation                 = serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "relation").item (0).getChildNodes ().item (0).getNodeValue ();
        Element  subjectDataID;
        
        try {
            subjectDataID = (Element) subjectIDNode.getElementsByTagNameNS (Constants.PSTRUCT_NS, "dataAccessor").item (0).getChildNodes ().item (0);
        } catch (NullPointerException noSubjDataId) {
            subjectDataID = null;
        }
        
        LinkedList objectIDs = new LinkedList ();
        for (int i = 0; i < objectIDNodes.getLength (); i++) {
            Element objectIDNode = (Element)objectIDNodes.item (i);
            
            Element objectDataID;
            try {
                objectDataID  = (Element) objectIDNode.getElementsByTagNameNS (Constants.PSTRUCT_NS, "dataAccessor").item (0).getChildNodes ().item (0);
            } catch (NullPointerException noObjDataId) {
                objectDataID = null;
            }
            
            String parameterName = objectIDNode.getElementsByTagNameNS (Constants.PSTRUCT_NS, "parameterName").item (0).getChildNodes ().item (0).getNodeValue ();
            GlobalPAssertionKey gpid = this.getGlobalPAssertionID (objectIDNode);
            
            Link link;
            try {
                Node locationNode = ((Element) serialised.getElementsByTagNameNS (Constants.PSTRUCT_NS, "objectLink").item (0)).getElementsByTagNameNS (Constants.PSTRUCT_NS, "provenanceStoreRef").item (0);
                if (locationNode != null) {
                    link = new Link (DOMWriter.writeToString (locationNode));
                } else {
                    link = null;
                }
            } catch (NullPointerException noLink) {
                link = null;
            }
            
            ObjectID oi = new ObjectID (gpid, parameterName, objectDataID, link);
            objectIDs.add (oi);
        }
        
        return new Relationship (localPAssertionID, subjectLocalPAssertionID, subjectDataID, subjectParameterName, relation, objectIDs);
    }
    
    public int getNumberOfExpectedMsgs (Element serialised) {
        String numStr = serialised.getElementsByTagNameNS (Constants.PRECORD_NS, "submissionFinished").item (0).getChildNodes ().item (0).getTextContent ();
        return Integer.parseInt (numStr.trim ());
    }
}
