package org.pasoa.dommap;

import java.util.Iterator;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class DOMDeserialiser {
    private String _namespace;
    
    public DOMDeserialiser (String namespace) {
        _namespace = namespace;
    }
    
    public DOMDeserialiser () {
        this (null);
    }
    
    public Element element (Node current) {
        NodeList children = current.getChildNodes ();
        
        for (int index = 0; index < children.getLength (); index += 1) {
            if (children.item (index).getNodeType () == Node.ELEMENT_NODE) {
                return (Element) children.item (index);
            }
        }
        
        return null;
    }
    
    private Element getElement (Node current) {
        if (current instanceof Document) {
            return ((Document) current).getDocumentElement ();
        } else {
            return (Element) current;
        }
    }
    
    public Iterator iterator (Node current, String childName) {
        return iterator (current, _namespace, childName);
    }

    public Iterator iterator (Node current, String namespace, String childName) {
        return new NodeListIterator (getElement (current).getElementsByTagNameNS (namespace, childName));
    }
    
    public DOMDeserialiser namespace (String namespace) {
        _namespace = namespace;
        return this;
    }
    
    public Element single (Node current, String childName) {
        NodeList list;
        
        if (current == null) {
            return null;
        }
        list = getElement (current).getElementsByTagNameNS (_namespace, childName);
        if (list.getLength () >= 1) {
            return (Element) list.item (0);
        } else {
            return null;
        }
    }
    
    public String text (Node current, String childName) {
        Element single = single (current, childName);
        
        if (single != null) {
            return single.getTextContent ();
        } else {
            return null;
        }
    }
}
