package org.pasoa.dommap;

import java.util.HashMap;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class DOMSerialiser {
    private Node   _root;
    private String _namespace;
    private Node   _current;
    
    public DOMSerialiser (Node root) {
        _root      = root;
        _namespace = root.getNamespaceURI ();
        _current   = root;
    }
    
    public DOMSerialiser () {
        this (new SimpleDocument (new HashMap ()));
    }
    
    public DOMSerialiser append (Node child) {
        if (child == null) {
            return this;
        }
        if (child instanceof Document) {
            append (((Document) child).getDocumentElement ());
        } else {
            _current.appendChild (child);
        }
        return this;
    }
    
    public DOMSerialiser copy (Element child) {
        return append (new SimpleElement (_root.getOwnerDocument (), (SimpleNode) _current, _current.getChildNodes ().getLength (), child));
    }
    
    public DOMSerialiser end () {
        _current = _current.getParentNode ();
        return this;
    }
    
    public Node getRoot () {
        return _root;
    }
    
    public DOMSerialiser namespace (String newNamespace) {
        _namespace = newNamespace;
        return this;
    }
    
    public DOMSerialiser namespace (String newNamespace, String prefix) {
        _namespace = newNamespace;
        ((SimpleNode) _current).addNamespace (prefix, _namespace);
        return this;
    }
    
    public DOMSerialiser single (String elementName) {
        start (elementName);
        return end ();
    }
    
    public DOMSerialiser single (String elementName, Node child) {
        if (child != null) {
            start (elementName);
            append (child);
            end ();
        }
        return this;
    }
    
    public DOMSerialiser single (String elementName, String content) {
        if (content != null) {
            single (elementName, new SimpleText (_root.getOwnerDocument (), content));
        }
        return this;
    }
    
    public DOMSerialiser single (String elementName, int content) {
        return single (elementName, Integer.toString (content));
    }
    
    public DOMSerialiser start (String elementName) {
        SimpleElement newElem = new SimpleElement (_root.getOwnerDocument (), _namespace, elementName);
        _current.appendChild (newElem);
        _current = newElem;
        return this;
    }
}
