package org.pasoa.dommap;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;

public abstract class MappedDocument extends SimpleDocument {
    protected MappedDocument (Map initialNamespaces) {
        super (initialNamespaces);
    }
    
    protected SimpleAttr buildAttribute (Node parent, String qname, String value) {
        return new SimpleAttr (this, parent, getNamespaceForPrefix (getPrefix (qname)), getLocalName (qname), value);
    }
    
    protected Node buildElement (SimpleNode parent, int childIndex, String qname, Map context) throws UnderlyingModelException {
        SimpleElement newElement = new SimpleElement (this, parent, childIndex, getNamespaceForPrefix (getPrefix (qname)), getLocalName (qname));
        Iterator      eachAttr   = getAttributes (qname, parent, context).iterator ();
        Attr          attr;
        
        while (eachAttr.hasNext ()) {
            attr = (Attr) eachAttr.next ();
            if (attr != null) {
                newElement.setAttributeNodeNS (attr);
            }
        }
        newElement.setContent (new LazyList (this, newElement, qname, context));
        
        return newElement;
    }
    
    protected SimpleText buildText (SimpleNode parent, int childIndex, String value) {
        return new SimpleText (this, parent, childIndex, value);
    }
    
    protected abstract List getAttributes (String parentName, Node parent, Map context) throws UnderlyingModelException;
    
    protected abstract Node getChild (String parentName, SimpleNode parent, int index, Map context) throws UnderlyingModelException;
    
    private static String getLocalName (String name) {
        return name.substring (name.indexOf (":") + 1).trim ();
    }
    
    protected abstract int getNumberOfChildren (String name, Map context) throws UnderlyingModelException;
    
    private static String getPrefix (String name) {
        return name.substring (0, name.indexOf (":")).trim ();
    }
    
    protected abstract String getRootElement () throws UnderlyingModelException;
    
    protected void initialise (Map initialContext) throws UnderlyingModelException {
        appendChild (buildElement (null, -2, getRootElement (), initialContext));
    }
}
