package org.pasoa.dommap;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMConfiguration;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.w3c.dom.UserDataHandler;

public class SimpleDocument extends SimpleNode implements Document {
    private int nsIndex = 0;
    
    String newNamespace() {
        nsIndex += 1;
        return "ns" + nsIndex;
    }
    
    public SimpleDocument (Map prefixToSpace) {
        super (prefixToSpace);
    }
    
    public SimpleDocument (Document convertFrom) {
        this (new HashMap ());
        appendChild (new SimpleElement (this, this, 0, convertFrom.getDocumentElement ()));
    }
    
    public Attr createAttribute (String name) throws DOMException {
        return createAttributeNS ("", name);
    }
    
    public Attr createAttributeNS (String namespaceURI, String qualifiedName) throws DOMException {
        return new SimpleAttr (this, namespaceURI, qualifiedName, "");
    }
    
    public Element createElement (String tagName) throws DOMException {
        return createElementNS ("", tagName);
    }
    
    public Element createElementNS (String namespaceURI, String qualifiedName) throws DOMException {
        return new SimpleElement (this, namespaceURI, qualifiedName);
    }
    
    public Element getDocumentElement () {
        return (Element) getChildNodes ().item (0);
    }
    
    public DocumentType getDoctype () {
        return null;
    }
    
    public DOMImplementation getImplementation () {
        return null;
    }
    
    public String getLocalName () {
        return null;
    }
    
    public String getNamespaceURI () {
        return null;
    }
    
    public short getNodeType () {
        return Node.DOCUMENT_NODE;
    }
    
    
    public Document getOwnerDocument () {
        return this;
    }
    
    public Node importNode (Node importedNode, boolean deep) throws DOMException {
//        System.out.println ("Called importNode");
/*        switch (importedNode.getNodeType ()) {
            case Node.ELEMENT_NODE:
                return createElementNS (importedNode.getNamespaceURI (), importedNode.getLocalName ());
        }*/
        throw new UnsupportedOperationException ();
    }
    
    /* Non-implemented methods */
    
    public void setXmlStandalone (boolean xmlStandalone) throws DOMException {
        System.out.println ("Called setXmlStandalone");
        throw new UnsupportedOperationException (); }
    
    public void setStrictErrorChecking (boolean strictErrorChecking) {
        System.out.println ("Called setStrictErrorChecking");
        throw new UnsupportedOperationException (); }
    
    public SimpleNode cloneNode (boolean deep, Document newDocument) {
        System.out.println ("Called cloneNode");
        throw new UnsupportedOperationException (); }
    
    public Object setUserData (String key, Object data, UserDataHandler handler) {
        System.out.println ("Called setUserData");
        throw new UnsupportedOperationException (); }
    
    public Node renameNode (Node n, String namespaceURI, String qualifiedName) throws DOMException {
        System.out.println ("Called renameNode");
        throw new UnsupportedOperationException (); }
    
    public void setXmlVersion (String xmlVersion) throws DOMException {
        System.out.println ("Called setXmlVersion");
        throw new UnsupportedOperationException (); }
    
    public void setTextContent (String textContent) throws DOMException {
        System.out.println ("Called setTextContent");
        throw new UnsupportedOperationException (); }
    
    public void setPrefix (String prefix) throws DOMException {
        System.out.println ("Called setPrefix");
        throw new UnsupportedOperationException (); }
    
    public void setNodeValue (String nodeValue) throws DOMException {
        System.out.println ("Called setNodeValue");
        throw new UnsupportedOperationException (); }
    
    public void setDocumentURI (String documentURI) {
        System.out.println ("Called setDocumentURI");
        throw new UnsupportedOperationException (); }
    
    public String lookupPrefix (String namespaceURI) {
        System.out.println ("Called lookupPrefix");
        throw new UnsupportedOperationException (); }
    
    public String lookupNamespaceURI (String prefix) {
        System.out.println ("Called lookupNamespaceURI");
        throw new UnsupportedOperationException (); }
    
    public boolean isDefaultNamespace (String namespaceURI) {
        System.out.println ("Called isDefaultNamespace");
        throw new UnsupportedOperationException (); }
    
    public Element getElementById (String elementId) {
        System.out.println ("Called getElementById");
        throw new UnsupportedOperationException (); }
    
    public Text createTextNode (String data) {
        System.out.println ("Called createTextNode");
        throw new UnsupportedOperationException (); }
    
    public CDATASection createCDATASection (String data) throws DOMException {
        System.out.println ("Called createCDATASection");
        throw new UnsupportedOperationException (); }
    
    public Comment createComment (String data) {
        System.out.println ("Called createComment");
        throw new UnsupportedOperationException (); }
    
    public EntityReference createEntityReference (String name) throws DOMException {
        System.out.println ("Called createEntityReference");
        throw new UnsupportedOperationException (); }
    
    public NodeList getElementsByTagName (String tagname) {
        System.out.println ("Called getElementsByTagName");
        throw new UnsupportedOperationException (); }
    
    public Object getUserData (String key) {
        System.out.println ("Called getUserData");
        throw new UnsupportedOperationException (); }
    
    public Node removeChild (Node oldChild) throws DOMException {
        System.out.println ("Called removeChild");
        throw new UnsupportedOperationException (); }
    
    public boolean isSameNode (Node other) {
        System.out.println ("Called isSameNode");
        throw new UnsupportedOperationException (); }
    
    public boolean isEqualNode (Node arg) {
        System.out.println ("Called isEqualNode");
        throw new UnsupportedOperationException (); }
    
    public Node adoptNode (Node source) throws DOMException {
        System.out.println ("Called adoptNode");
        throw new UnsupportedOperationException (); }
    
    public short compareDocumentPosition (Node other) throws DOMException {
        System.out.println ("Called compareDocumentPosition");
        throw new UnsupportedOperationException (); }
    
    public Node replaceChild (Node newChild, Node oldChild) throws DOMException {
        System.out.println ("Called replaceChild");
        throw new UnsupportedOperationException (); }
    
    public void normalizeDocument () {
        System.out.println ("Called normalizeDocument");
        throw new UnsupportedOperationException (); }
    
    public void normalize () {
        System.out.println ("Called normalize");
        throw new UnsupportedOperationException (); }
    
    public boolean isSupported (String feature, String version) {
        System.out.println ("Called isSupported");
        throw new UnsupportedOperationException (); }
    
    public Node insertBefore (Node newChild, Node refChild) throws DOMException {
        System.out.println ("Called insertBefore");
        throw new UnsupportedOperationException (); }
    
    public DOMConfiguration getDomConfig () {
        System.out.println ("Called getDomConfig");
        throw new UnsupportedOperationException (); }
    
    public String getDocumentURI () {
        System.out.println ("Called getDocumentURI");
        throw new UnsupportedOperationException (); }
    
    public String getBaseURI () {
        System.out.println ("Called getBaseURI");
        throw new UnsupportedOperationException (); }
    
    public NamedNodeMap getAttributes () {
        System.out.println ("Called getAttributes");
        throw new UnsupportedOperationException (); }
    
    public ProcessingInstruction createProcessingInstruction (String target, String data) throws DOMException {
        System.out.println ("Called createProcessingInstruction");
        throw new UnsupportedOperationException (); }
    
    public DocumentFragment createDocumentFragment () {
        System.out.println ("Called createDocumentFragment");
        throw new UnsupportedOperationException (); }
    
    public NodeList getElementsByTagNameNS (String namespaceURI, String localName) {
        System.out.println ("Called getElementsByTagNameNS");
        throw new UnsupportedOperationException (); }
    
    public Object getFeature (String feature, String version) {
        System.out.println ("Called getFeature");
        throw new UnsupportedOperationException (); }
    
    public String getInputEncoding () {
        System.out.println ("Called getInputEncoding");
        throw new UnsupportedOperationException (); }
    
    public String getNodeValue () throws DOMException {
        System.out.println ("Called getNodeValue");
        throw new UnsupportedOperationException (); }
    
    public Node getPreviousSibling () {
        System.out.println ("Called getPreviousSibling");
        throw new UnsupportedOperationException (); }
    
    public boolean getStrictErrorChecking () {
        System.out.println ("Called getStrictErrorChecking");
        throw new UnsupportedOperationException (); }
    
    public String getTextContent () throws DOMException {
        System.out.println ("Called getTextContent");
        throw new UnsupportedOperationException (); }
    
    public String getXmlEncoding () {
        System.out.println ("Called getXmlEncoding");
        throw new UnsupportedOperationException (); }
    
    public boolean getXmlStandalone () {
        System.out.println ("Called getXmlStandalone");
        throw new UnsupportedOperationException (); }
    
    public String getXmlVersion () {
        System.out.println ("Called getXmlVersion");
        throw new UnsupportedOperationException (); }
    
    public boolean hasAttributes () {
        System.out.println ("Called hasAttributes");
        throw new UnsupportedOperationException (); }
}
