package org.pasoa.dommap;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

public class SimpleElement extends SimpleNode implements Element {
    public SimpleElement (Document owner, String elementNS, String name) {
        super (owner, elementNS, name);
    }
    
    public SimpleElement (Document owner, SimpleNode parent, int childIndex, String elementNS, String name) {
        super (owner, elementNS, name);
        if (parent != null) {
            parent.incorporateChild (this, childIndex);
        }
    }
    
    public SimpleElement (Document owner, SimpleNode parent, int childIndex, Element convertFrom) {
        this (owner, parent, childIndex, convertFrom.getNamespaceURI (), convertFrom.getLocalName ());
        NodeList     eachNode = convertFrom.getChildNodes ();
        NamedNodeMap eachAttr = convertFrom.getAttributes ();
        Attr         space;
        
/*        for (int index = 0; index < eachAttr.getLength (); index += 1) {
            space = (Attr) eachAttr.item (index);
            if (space.getNodeName ().startsWith ("xmlns:")) {
                processNamespace (space);
            }
        }
        for (int index = 0; index < eachNode.getLength (); index += 1) {
            if (eachNode.item (index).getNodeType () == Node.ATTRIBUTE_NODE) {
                space = (Attr) eachNode.item (index);
                if (space.getNodeName ().startsWith ("xmlns:")) {
                    processNamespace (space);
                }
            }
        }*/
        
        if (getPrefix () == null && owner instanceof SimpleDocument && owner != null && convertFrom.getNamespaceURI () != null) {
            addNamespace (((SimpleDocument) owner).newNamespace (), convertFrom.getNamespaceURI ());
        }
        
        eachNode = convertFrom.getChildNodes ();
        for (int index = 0; index < eachNode.getLength (); index += 1) {
            if (eachNode.item (index).getNodeType () == Node.ATTRIBUTE_NODE) {
                space = (Attr) eachNode.item (index);
                if (!isNamespaceDeclaration (space)) {
                    appendChild (new SimpleAttr (owner, this, (Attr) eachNode.item (index)));
                }
            }
            if (eachNode.item (index).getNodeType () == Node.ELEMENT_NODE) {
                appendChild (new SimpleElement (owner, this, index, (Element) eachNode.item (index)));
            }
            if (eachNode.item (index).getNodeType () == Node.TEXT_NODE) {
                appendChild (new SimpleText (owner, this, index, (Text) eachNode.item (index)));
            }
        }
        
        eachAttr = convertFrom.getAttributes ();
        for (int index = 0; index < eachAttr.getLength (); index += 1) {
            space = (Attr) eachAttr.item (index);
            if (!isNamespaceDeclaration (space)) {
                appendChild (new SimpleAttr (owner, this, (Attr) eachAttr.item (index)));
            }
        }
    }
    
    private boolean isNamespaceDeclaration (Attr space) {
        return (space.getNodeName () != null && space.getNodeName ().startsWith ("xmlns:"))
        || (space.getPrefix () != null && space.getPrefix ().equals ("xmlns"));
    }
    
    public Node appendChild (Node newChild) throws DOMException {
        if (newChild instanceof Attr) {
            super.setAttributeNS (newChild.getNamespaceURI (), newChild.getLocalName (), newChild.getNodeValue ());
            return newChild;
        }
        if (newChild instanceof Element || newChild instanceof Text) {
            return super.appendChild (newChild);
        } else {
            throw new DOMException (DOMException.HIERARCHY_REQUEST_ERR, "Cannot add node of type " + newChild.getNodeType () + " to element");
        }
    }
    
    public Node appendText (String newText) throws DOMException {
        return appendChild (new SimpleText (getOwnerDocument (), newText));
    }
    
    public SimpleNode cloneNode (boolean deep, Document newDocument) {
        SimpleElement cloned;
        Iterator      eachClone;
        Iterator      eachAttr;
        Attr          attribute;
        
        if (!deep) {
            throw new UnsupportedOperationException ();
        }
        cloned = new SimpleElement (newDocument, getNamespaceURI (), getLocalName ());
        eachClone = cloneChildren (newDocument).iterator ();
        while (eachClone.hasNext ()) {
            cloned.appendChild ((Node) eachClone.next ());
        }
        eachAttr = ((SimpleNamedNodeMap) getAttributes ()).iterator ();
        while (eachAttr.hasNext ()) {
            attribute = (Attr) eachAttr.next ();
            cloned.setAttributeNS (attribute.getNamespaceURI (), attribute.getLocalName (), attribute.getNodeValue ());
        }
        
        return cloned;
    }
    
    public NodeList getElementsByTagNameNS (String namespaceURI, String localName) throws DOMException {
        NodeList children = getChildNodes ();
        List     found    = new LinkedList ();
        Node     child;
        
        for (int index = 0; index < children.getLength (); index += 1) {
            child = children.item (index);
            if (child.getNamespaceURI () != null && child.getNamespaceURI ().equals (namespaceURI)
            && child.getLocalName () != null && child.getLocalName ().equals (localName)) {
                found.add (child);
            }
        }
        
        return new SimpleNodeList (found);
    }
    
    public short getNodeType () {
        return Node.ELEMENT_NODE;
    }
    
    public String getNodeValue () {
        return null;
    }

    public String getTextContent () throws DOMException {
        StringBuffer content  = new StringBuffer ();
        NodeList     children = getChildNodes ();
        
        for (int index = 0; index < children.getLength (); index += 1) {
            content.append (children.item (index).getTextContent ());
        }
        
        return content.toString ();
    }
    
    private void processNamespace (Attr space) {
        String name   = space.getNodeName ();
        String prefix = name.substring (name.indexOf (":") + 1);
        String ns     = space.getNodeValue ();
        
        addNamespace (prefix, ns);
    }
}
