package org.pasoa.dommap;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

public class SimpleText extends SimpleNode implements Text {
    private String text;
    
    public SimpleText (Document owner, String content) {
        super (owner, "", "#text");
        text = content;
    }
    
    public SimpleText (Document owner, SimpleNode parent, int childIndex, String content) {
        super (owner, "", "#text");
        text = content;
        if (parent != null) {
            parent.incorporateChild (this, childIndex);
        }
    }
    
    public SimpleText (Document owner, SimpleNode parent, int childIndex, Text convertFrom) {
        this (owner, parent, childIndex, convertFrom.getNodeValue ());
    }
    
    public SimpleNode cloneNode (boolean deep, Document newOwner) {
        return new SimpleText (newOwner, text);
    }
    
    public String getData () throws DOMException {
        return getNodeValue ();
    }

    public String getNodeName () {
        return "#text";
    }
    
    public short getNodeType () {
        return Node.TEXT_NODE;
    }
    
    public String getNodeValue () {
        return text;
    }
    
    public String getTextContent () throws DOMException {
        return getNodeValue ();
    }
    
    /* Non-implemented methods */
    
    public String getWholeText () {
        throw new UnsupportedOperationException ();
    }
    
    public Text replaceWholeText (String content) throws DOMException {
        throw new UnsupportedOperationException ();
    }
    
    public boolean isElementContentWhitespace () {
        throw new UnsupportedOperationException ();
    }
    
    public boolean isEqualNode (Node other) {
        return other.getNodeType () == getNodeType ()
        && getNodeValue ().equals (other.getNodeValue ())
        && getChildNodes ().getLength () == other.getChildNodes ().getLength ();
    }
    
    public Text splitText (int offset) throws DOMException {
        throw new UnsupportedOperationException ();
    }
    
    public void replaceData (int offset, int count, String arg) throws DOMException {
        throw new UnsupportedOperationException ();
    }
    
    public void deleteData (int offset, int count) throws DOMException {
        throw new UnsupportedOperationException ();
    }
    
    public void insertData (int offset, String arg) throws DOMException {
        throw new UnsupportedOperationException ();
    }
    
    public void appendData (String arg) throws DOMException {
        throw new UnsupportedOperationException ();
    }
    
    public String substringData (int offset, int count) throws DOMException {
        throw new UnsupportedOperationException ();
    }
    
    public int getLength () {
        throw new UnsupportedOperationException ();
    }
    
    public void setData (String data) throws DOMException {
        throw new UnsupportedOperationException ();
    }
}
