package org.pasoa.factory;

import org.pasoa.processor.PlugInRegistry;
import org.pasoa.query.provenance.ProvenanceQueryPlugin;
import org.pasoa.query.xquery.XQueryPlugIn;
import org.pasoa.record.RecordPlugIn;
import org.pasoa.storage.BackendStore;
import org.pasoa.storage.GeneralStorageSystem;
import org.pasoa.storage.StorageSystem;
import org.pasoa.storage.je.BerkeleyDBJEStorage;

public class ProvenanceStoreFactory
{
    public static ProvenanceStore newInstance (String storagePath, long cacheWaitTime)
    {   
        BackendStore backend = new BerkeleyDBJEStorage (storagePath);
        
        StorageSystem storageSystem = new GeneralStorageSystem (backend, "org.pasoa.query.xquery.ipsixq.XQueryUtilities");
        
        RecordPlugIn rpi = new RecordPlugIn ();
        XQueryPlugIn xqpi = new XQueryPlugIn ();
        ProvenanceQueryPlugin pqpi = new ProvenanceQueryPlugin ();
        
        PlugInRegistry pir = new PlugInRegistry ();
        try {
            pir.registerPlugIn ("/record", rpi);
            pir.registerPlugIn ("/xquery", xqpi);
            pir.registerPlugIn ("/pquery", pqpi);
        } catch (Exception e) {
            System.out.println ("Unable to register plug-ins");
            return null;
        }
        
        ProvenanceStore pStore = new ProvenanceStore (pir, storageSystem, cacheWaitTime);
        
        if (pStore == null) {
            System.out.println ("Factory: pStore is null");
        }
        
        return pStore;
    }
}
