package org.pasoa.query.dom;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import org.pasoa.common.Constants;
import org.pasoa.dommap.MappedDocument;
import org.pasoa.dommap.SimpleElement;
import org.pasoa.dommap.SimpleNode;
import org.pasoa.dommap.UnderlyingModelException;
import org.pasoa.common.Retrieval;
import org.pasoa.common.structure.ActorStatePAssertion;
import org.pasoa.common.structure.InteractionPAssertion;
import org.pasoa.common.structure.InteractionRecord;
import org.pasoa.common.structure.ObjectID;
import org.pasoa.common.structure.PAssertion;
import org.pasoa.common.structure.Relationship;
import org.pasoa.common.structure.SubjectID;
import org.pasoa.common.structure.View;
import org.pasoa.dommap.DOMWriter;
import org.w3c.dom.Node;

public class QStructureActivityDocument extends MappedDocument {
    private Retrieval basic;
    private Map       childQuantities;
    
    private QStructureActivityDocument (Retrieval store, Map context, Map namespaces) throws Exception {
        super (namespaceMap (namespaces));
        basic = store;
        initialise (context);
        childQuantities = new HashMap ();
        
        // These define the number of guaranteed children for each element
        // Optional children will be detected and declared in getNumberOfChildren()
        childQuantities.put ("pasoa:interactionRecord",          new Integer (1));
        childQuantities.put ("pasoa:interactionKey",             new Integer (3));
        childQuantities.put ("pasoa:messageSink",                new Integer (1));
        childQuantities.put ("pasoa:messageSource",              new Integer (1));
        childQuantities.put ("pasoa:interactionId",              new Integer (1));
        childQuantities.put ("pasoa:sender",                     new Integer (1));
        childQuantities.put ("pasoa:receiver",                   new Integer (1));
        childQuantities.put ("pasoa:inferred",                   new Integer (1));
        childQuantities.put ("pasoa:asserter",                   new Integer (1));
        childQuantities.put ("pasoa:interactionPAssertion",      new Integer (2));
        childQuantities.put ("pasoa:actorStatePAssertion",       new Integer (1));
        childQuantities.put ("pasoa:relationshipPAssertion",     new Integer (3));
        childQuantities.put ("pasoa:content",                    new Integer (1));
        childQuantities.put ("pasoa:subjectId",                  new Integer (2));
        childQuantities.put ("pasoa:objectId",                   new Integer (4));
        childQuantities.put ("pasoa:dataAccessor",               new Integer (1));
        childQuantities.put ("pasoa:objectLink",                 new Integer (1));
        childQuantities.put ("pasoa:viewLink",                   new Integer (1));
        childQuantities.put ("pasoa:localPAssertionId",          new Integer (1));
        childQuantities.put ("pasoa:relation",                   new Integer (1));
        childQuantities.put ("pasoa:viewKind",                   new Integer (1));
        childQuantities.put ("pasoa:parameterName",              new Integer (1));
        childQuantities.put ("pasoa:numberOfExpectedAssertions", new Integer (1));
    }
    
    public QStructureActivityDocument (Retrieval store, InteractionRecord exchange, Map namespaces) throws Exception {
        this (store, exchangeMap (exchange), namespaces);
    }
    
    protected QStructureActivityDocument (Retrieval store, Map namespaces) throws Exception {
        this (store, new HashMap (), namespaces);
    }
    
    protected Retrieval getBasic () {
        return basic;
    }
    
    protected List getAttributes (String name, Node parent, Map context) throws UnderlyingModelException {
        return new LinkedList ();
    }
    
    protected Node getChild (String name, SimpleNode parent, int index, Map context) throws UnderlyingModelException {
//System.out.println ("get child " + index + " of " + name);
        InteractionRecord exchange = (InteractionRecord) context.get ("exchange");
        View              view     = (View) context.get ("view");
        int               modIndex = index;
        Map               newContext;

        if (name.equals ("pasoa:interactionRecord")) {
            newContext = new HashMap (context);
            if (modIndex >= 1 && exchange.getSenderView () == null) {
                modIndex += 1;
            }
            switch (modIndex) {
                case 0:
                    return buildElement (parent, index, "pasoa:interactionKey", context);
                case 1:
                    newContext.put ("view", exchange.getSenderView ());
                    return buildElement (parent, index, "pasoa:sender", newContext);
                case 2:
                    newContext.put ("view", exchange.getReceiverView ());
                    return buildElement (parent, index, "pasoa:receiver", newContext);
            }
            newContext.put ("view", exchange.getInferredViews ().get (modIndex - 3));
            return buildElement (parent, index, "pasoa:inferred", newContext);
        }
        if (name.equals ("pasoa:interactionKey")) {
            switch (modIndex) {
                case 0:
                    return buildElement (parent, index, "pasoa:messageSource", context);
                case 1:
                    return buildElement (parent, index, "pasoa:messageSink", context);
                case 2:
                    return buildElement (parent, index, "pasoa:interactionId", context);
            }
        }
        if (name.equals ("pasoa:messageSink")) {
            if (context.get ("objectID") == null) {
                return new SimpleElement (this, parent, index, exchange.getInteractionKey ().getMessageSink ());
            } else {
                return new SimpleElement (this, parent, index, ((ObjectID) context.get ("objectID")).getGlobalPAssertionKey ().getInteractionKey ().getMessageSink ());
            }
        }
        if (name.equals ("pasoa:messageSource")) {
            if (context.get ("objectID") == null) {
                return new SimpleElement (this, parent, index, exchange.getInteractionKey ().getMessageSource ());
            } else {
                return new SimpleElement (this, parent, index, ((ObjectID) context.get ("objectID")).getGlobalPAssertionKey ().getInteractionKey ().getMessageSource ());
            }
        }
        if (name.equals ("pasoa:interactionId")) {
            if (context.get ("objectID") == null) {
                return buildText (parent, index, exchange.getInteractionKey ().getInteractionID ());
            } else {
                return buildText (parent, index, ((ObjectID) context.get ("objectID")).getGlobalPAssertionKey ().getInteractionKey ().getInteractionID ());
            }
        }
        if (name.equals ("pasoa:sender") || name.equals ("pasoa:receiver") || name.equals ("pasoa:inferred")) {
            newContext = new HashMap (context);
            if (modIndex == 0) {
                return buildElement (parent, index, "pasoa:asserter", newContext);
            }
            modIndex -= 1;
            if (modIndex < view.getInteractionPAssertions ().size ()) {
                return new SimpleElement (this, parent, index, ((InteractionPAssertion) view.getInteractionPAssertions ().get (modIndex)).getMessageBody ());
            }
            modIndex -= view.getInteractionPAssertions ().size ();
            if (modIndex < view.getActorStatePAssertions ().size ()) {
                return new SimpleElement (this, parent, index, ((ActorStatePAssertion) view.getActorStatePAssertions ().get (modIndex)).getActorStateBody ());
            }
            modIndex -= view.getActorStatePAssertions ().size ();
            if (modIndex < view.getRelationships ().size ()) {
                newContext.put ("assertion", view.getRelationships ().get (modIndex));
                return buildElement (parent, index, "pasoa:relationshipPAssertion", newContext);
            }
            modIndex -= view.getRelationships ().size ();
            if (view.getNumberOfExpectedPAssertions () < 0) {
                modIndex += 1;
            }
            switch (modIndex) {
                case 0:
                    return buildElement (parent, index, "pasoa:numberOfExpectedAssertions", newContext);
                case 1:
                    return buildElement (parent, index, "pasoa:viewLink", newContext);
            }
        }
        if (name.equals ("pasoa:numberOfExpectedAssertions")) {
            return buildText (parent, index, Integer.toString (((View) context.get ("view")).getNumberOfExpectedPAssertions ()));
        }
        if (name.equals ("pasoa:viewLink")) {
            return buildElement (parent, index, "pasoa:provenanceStoreRef", context);
        }
        if (name.equals ("pasoa:asserter")) {
            return new SimpleElement (this, parent, index, ((View) context.get ("view")).getAsserter ());
        }
        if (name.equals ("pasoa:documentationStyle")) {
            InteractionPAssertion assertion = (InteractionPAssertion) context.get ("assertion");
            return buildText (parent, index, assertion.getDocumentationStyle ());
        }
        if (name.equals ("pasoa:relationshipPAssertion")) {
            Relationship assertion = (Relationship) context.get ("assertion");
            newContext = new HashMap (context);
            switch (modIndex) {
                case 0:
                    return buildElement (parent, index, "pasoa:localPAssertionId", newContext);
                case 1:
                    newContext.put ("subjectID", assertion.getSubject ());
                    return buildElement (parent, index, "pasoa:subjectId", newContext);
                case 2:
                    return buildElement (parent, index, "pasoa:relation", newContext);
            }
            newContext.put ("objectID", assertion.getObjectIDs ().get (modIndex - 3));
            return buildElement (parent, index, "pasoa:objectId", newContext);
        }
        if (name.equals ("pasoa:localPAssertionId")) {
            if (context.get ("objectID") != null) {
                return buildText (parent, index, ((ObjectID) context.get ("objectID")).getGlobalPAssertionKey ().getLocalPAssertionID ());
            }
            if (context.get ("subjectID") != null) {
                return buildText (parent, index, ((SubjectID) context.get ("subjectID")).getLocalPAssertionID ());
            }
            return buildText (parent, index, ((PAssertion) context.get ("assertion")).getLocalPAssertionID ());
        }
        if (name.equals ("pasoa:relation")) {
            Relationship assertion = (Relationship) context.get ("assertion");
            return buildText (parent, index, assertion.getRelation ());
        }
        if (name.equals ("pasoa:subjectId")) {
            SubjectID subject = (SubjectID) context.get ("subjectID");
            if (modIndex >= 1 && subject.getDataAccessor () == null) {
                modIndex += 1;
            }
            switch (modIndex) {
                case 0:
                    return buildElement (parent, index, "pasoa:localPAssertionId", context);
                case 1:
                    return buildElement (parent, index, "pasoa:dataAccessor",      context);
                case 2:
                    return buildElement (parent, index, "pasoa:parameterName",     context);
            }
        }
        if (name.equals ("pasoa:objectId")) {
            ObjectID objectID = (ObjectID) context.get ("objectID");
            if (modIndex >= 3 && objectID.getObjectDataAccessor () == null) {
                modIndex += 1;
            }
            switch (modIndex) {
                case 0:
                    return buildElement (parent, index, "pasoa:interactionKey",    context);
                case 1:
                    return buildElement (parent, index, "pasoa:viewKind",          context);
                case 2:
                    return buildElement (parent, index, "pasoa:localPAssertionId", context);
                case 3:
                    return buildElement (parent, index, "pasoa:dataAccessor",      context);
                case 4:
                    return buildElement (parent, index, "pasoa:parameterName",     context);
                case 5:
                    return buildElement (parent, index, "pasoa:objectLink",        context);
            }
        }
        if (name.equals ("pasoa:dataAccessor")) {
            if (context.get ("subjectID") != null) {
                return ((SubjectID) context.get ("subjectID")).getDataAccessor ();
            } else {
                return ((ObjectID) context.get ("objectID")).getObjectDataAccessor ();
            }
        }
        if (name.equals ("pasoa:viewKind")) {
            ObjectID objectID = (ObjectID) context.get ("objectID");
            return buildText (parent, index, objectID.getGlobalPAssertionKey ().getViewName ());
        }
        if (name.equals ("pasoa:objectLink")) {
            return buildElement (parent, index, "pasoa:provenanceStoreRef", context);
        }
        if (name.equals ("pasoa:provenanceStoreRef")) {
            ObjectID objectID = (ObjectID) context.get ("objectID");
            if (objectID == null) {
                return buildText (parent, index, ((View) context.get ("view")).getViewLink ().getStoreURL ());
            } else {
                return buildText (parent, index, objectID.getLink ().getStoreURL ());
            }
        }
        if (name.equals ("pasoa:parameterName")) {
            if (context.get ("subjectID") != null) {
                return buildText (parent, index, ((SubjectID) context.get ("subjectID")).getParameterName ());
            } else {
                return buildText (parent, index, ((ObjectID) context.get ("objectID")).getParameterName ());
            }
        }

        throw new NoSuchElementException ();
    }
    
    protected int getNumberOfChildren (String name, Map context) {
        InteractionRecord exchange  = (InteractionRecord) context.get ("exchange");
        View              view      = null;
        Integer           mandatory = (Integer) childQuantities.get (name);
        int               plus      = 0;
        
        if (mandatory == null) {
            mandatory = new Integer (0);
        }
        
        if (name.equals ("pasoa:interactionRecord")) {
            if (exchange.getSenderView () != null) {
                plus += 1;
            }
            if (exchange.getReceiverView () != null) {
                plus += 1;
            }
            plus += exchange.getInferredViews ().size ();
        }
        if (name.equals ("pasoa:sender") || name.equals ("pasoa:receiver") || name.equals ("pasoa:inferred")) {
            if (name.equals ("pasoa:sender")) {
                view = exchange.getSenderView ();
            }
            if (name.equals ("pasoa:receiver")) {
                view = exchange.getReceiverView ();
            }
            if (name.equals ("pasoa:inferred")) {
                view = (View) context.get ("inferred");
            }
            if (view.getNumberOfExpectedPAssertions () >= 0) {
                plus += 1;
            }
            if (view.getViewLink () != null) {
                plus += 1;
            }
            plus += view.getInteractionPAssertions ().size () + view.getActorStatePAssertions ().size () + view.getRelationships ().size ();
        }
        if (name.equals ("pasoa:relationshipPAssertion")) {
            Relationship assertion = (Relationship) context.get ("assertion");
            plus = assertion.getObjectIDs ().size ();
        }
        if (name.equals ("pasoa:subjectId")) {
            SubjectID subject = (SubjectID) context.get ("subjectID");
            if (subject.getDataAccessor () != null) {
                plus = 1;
            }
        }
        if (name.equals ("pasoa:objectId")) {
            ObjectID objectID = (ObjectID) context.get ("objectID");
            if (objectID.getObjectDataAccessor () != null) {
                plus += 1;
            }
            if (objectID.getLink () != null) {
                plus += 1;
            }
        }
        
        return mandatory.intValue () + plus;
    }
    
    protected String getRootElement () {
        return "pasoa:interactionRecord";
    }
    
    protected static Map namespaceMap (Map additions) {
        Map newMap = new HashMap (additions);
        
        newMap.put ("pasoa", Constants.PSTRUCT_NS);
        
        return newMap;
    }
    
    private static Map exchangeMap (InteractionRecord exhange) {
        Map context = new HashMap ();
        context.put ("exchange", exhange);
        return context;
    }
}
