package org.pasoa.query.provenance;

import java.util.Arrays;
import java.util.List;
import org.pasoa.common.Constants;
import org.pasoa.common.structure.GlobalPAssertionKey;
import org.pasoa.common.structure.InteractionKey;
import org.pasoa.common.structure.PAssertionDataKey;
import org.pasoa.common.structure.deserialiser.PStructureDeserialiser;
import org.pasoa.dommap.DOMDeserialiser;
import org.pasoa.query.QueryProblem;
import org.pasoa.query.xpath.SingleNodeXPath;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class PAssertionDataItem {
    private Node _item;
    
    public PAssertionDataItem (Node item) {
        _item = item;
    }

    private Element getAncestorNode (String[] localNames) throws QueryProblem {
        List locals  = Arrays.asList (localNames);
        Node current = _item;

        while (current != null && !matchesName (locals, current.getNamespaceURI (), current.getLocalName (), current.getNodeName ())) {
            current = current.getParentNode ();
        }

        if (current == null) {
            throw new QueryProblem ("Cannot find ancestor node of name " + locals + " from current node " + _item.getNodeName ());
        }
        
        return (Element) current;
    }
    
    private boolean matchesName (List matches, String namespace, String local, String whole) {
        if (namespace != null && local != null) {
            return namespace.equals (Constants.PSTRUCT_NS) && matches.contains (local);
        }
        if (whole == null) {
            return false;
        }
        return matches.contains (whole.substring (whole.indexOf (":") + 1));
    }

    public Element getDataAccessor () throws QueryProblem {
        return getXPathDataAccessor ().toXML ();
    }
    
    public SingleNodeXPath getXPathDataAccessor () throws QueryProblem {
        return SingleNodeXPath.generate (_item, getAncestorNode (new String[] {"interactionPAssertion", "actorStatePAssertion", "relationship"}));
    }
    
    public InteractionKey getInteractionKey () throws QueryProblem {
        Element record = getAncestorNode (new String[] {"interactionRecord"});
        Element key    = new DOMDeserialiser (Constants.PSTRUCT_NS).single (record, "interactionKey");
        
        return new PStructureDeserialiser ().getInteractionRecordID (key);
    }
    
    public GlobalPAssertionKey getGlobalPAssertionKey () throws QueryProblem {
        return new GlobalPAssertionKey (getInteractionKey (), getViewKind (), getLocalPAssertionID ());
    }

    public String getLocalPAssertionID () throws QueryProblem {
        Element assertion = getAncestorNode (new String[] {"interactionPAssertion", "actorStatePAssertion", "relationship"});
        
        return new DOMDeserialiser (Constants.PSTRUCT_NS).text (assertion, "localPAssertionId");
    }

    public PAssertionDataKey getPAssertionDataKey () throws QueryProblem {
        return new PAssertionDataKey (getGlobalPAssertionKey (), getDataAccessor ());
    }
    
    public String getViewKind () throws QueryProblem {
        if (getAncestorNode (new String[] {"sender", "receiver"}).getLocalName ().equals ("sender")) {
            return Constants.CLIENT_VIEW;
        } else {
            return Constants.SERVICE_VIEW;
        }
    }
}
