package org.pasoa.query.provenance;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import org.pasoa.common.Constants;
import org.pasoa.common.Retrieval;
import org.pasoa.common.structure.ActorStatePAssertion;
import org.pasoa.common.structure.GlobalPAssertionKey;
import org.pasoa.common.structure.InteractionKey;
import org.pasoa.common.structure.InteractionPAssertion;
import org.pasoa.common.structure.InteractionRecord;
import org.pasoa.common.structure.ObjectID;
import org.pasoa.common.structure.PAssertion;
import org.pasoa.common.structure.PAssertionDataKey;
import org.pasoa.common.structure.Relationship;
import org.pasoa.common.structure.View;
import org.pasoa.dommap.DOMWriter;
import org.pasoa.query.QueryProblem;
import org.pasoa.query.dom.QStructureDocument;
import org.pasoa.query.xpath.SingleNodeXPath;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class ProvenanceQuery implements PStructureReference {
    private DocumentSetAccessorSearch _queryDataHandle;
    private DocumentCheck             _relationshipTargetCheck;
    private Retrieval                 _navigate;
    private String                    _storeURL;
    
    public ProvenanceQuery (Retrieval retrieval, String thisStoreURL, DocumentSetAccessorSearch queryDataHandle, DocumentCheck relationshipTargetCheck) {
        _queryDataHandle         = queryDataHandle;
        _relationshipTargetCheck = relationshipTargetCheck;
        _navigate                = retrieval;
        _storeURL                = thisStoreURL;
    }
    
    private InteractionRecord findInteractionRecord (InteractionKey irid) throws QueryProblem {
        InteractionRecord record;
        
        try {
            for (long index = 0; index < _navigate.getNumberOfInteractionRecords (); index += 1) {
                record = _navigate.getInteractionRecord (index);
                if (record.getInteractionKey ().equals (irid)) {
                    return record;
                }
            }
        } catch (Exception unspecifiedProblem) {
            throw new QueryProblem (unspecifiedProblem);
        }
        
        throw new QueryProblem ("Reference to unknown interaction: " + irid);
    }
    
    private Element findPAssertionBody (GlobalPAssertionKey gpak) throws QueryProblem {
        InteractionRecord interaction = findInteractionRecord (gpak.getInteractionKey ());
        View              view;
        Iterator          eachPA;
        PAssertion        assertion;
        
        if (gpak.getViewName ().equals (Constants.CLIENT_VIEW)) {
            view = interaction.getSenderView ();
        } else {
            view = interaction.getReceiverView ();
        }
        eachPA = view.getActorStatePAssertions ().iterator ();
        while (eachPA.hasNext ()) {
            assertion = (PAssertion) eachPA.next ();
            if (assertion.getLocalPAssertionID ().equals (gpak.getLocalPAssertionID ())) {
                return ((ActorStatePAssertion) assertion).getActorStateBody ();
            }
        }
        eachPA = view.getInteractionPAssertions ().iterator ();
        while (eachPA.hasNext ()) {
            assertion = (PAssertion) eachPA.next ();
            if (assertion.getLocalPAssertionID ().equals (gpak.getLocalPAssertionID ())) {
                return ((InteractionPAssertion) assertion).getMessageBody ();
            }
        }
        
        return null;
    }
    
    /**
     * Will be a p-structure representation of the results of the resolve() method.
     */
    public QStructureDocument getPStructure () {
        return null;
    }
    
    private Collection getRelationships (PAssertionDataKey subject) throws QueryProblem {
        InteractionRecord interaction = findInteractionRecord (subject.getGlobalPAssertionKey ().getInteractionKey ());
        Collection        bothViews   = new LinkedList ();
        SingleNodeXPath   padkAccess  = SingleNodeXPath.fromXML (subject.getDataAccessor ());
        SingleNodeXPath   relAccess;
        Iterator          eachRelate;
        Relationship      relation;
        
        eachRelate = interaction.getSenderView (). getRelationships ().iterator ();
        while (eachRelate.hasNext ()) {
            relation  = (Relationship) eachRelate.next ();
            relAccess = SingleNodeXPath.fromXML (relation.getSubject ().getDataAccessor ());
            if (relation.getSubject ().getLocalPAssertionID ().equals (subject.getGlobalPAssertionKey ().getLocalPAssertionID ()) &&
                    relAccess.equals (padkAccess)) {
                bothViews.add (relation);
            }
        }
        eachRelate = interaction.getReceiverView (). getRelationships ().iterator ();
        while (eachRelate.hasNext ()) {
            relation  = (Relationship) eachRelate.next ();
            relAccess = SingleNodeXPath.fromXML (relation.getSubject ().getDataAccessor ());
            if (relation.getSubject ().getLocalPAssertionID ().equals (subject.getGlobalPAssertionKey ().getLocalPAssertionID ()) &&
                    relAccess.equals (padkAccess)) {
                bothViews.add (relation);
            }
        }

        return bothViews;
    }
    
    private Collection getRelationshipTargets (Relationship relationship) throws QueryProblem {
        Collection targets    = new LinkedList ();
        Iterator   eachObject = relationship.getObjectIDs ().iterator ();
        ObjectID   object;
        
        while (eachObject.hasNext ()) {
            object = (ObjectID) eachObject.next ();
            targets.add (new RelationshipTarget (object, relationship.getRelation (), findPAssertionBody (object.getGlobalPAssertionKey ())));
        }
        
        return targets;
    }
    
    /**
     * @return A collection of FullRelationships
     */
    public ProvenanceQueryResults resolve () throws QueryProblem {
        ProvenanceQueryResults results = resolve (_queryDataHandle.resolve (), new HashSet ());
        
System.out.println ("Starts:        " + results.getStarts ().size ());
System.out.println ("Relationships: " + results.getFullRelationships ().size ());
        return results;
    }
    
    private ProvenanceQueryResults resolve (Collection starts, Collection excludes) throws QueryProblem {
        Collection         results   = new HashSet ();
        Iterator           eachStart = starts.iterator ();
        Collection         next      = new HashSet ();
        Collection         keyStarts = new LinkedList ();
        Object             item;
        PAssertionDataKey  padk;
        Iterator           eachRelation;
        Iterator           eachTarget;
        Relationship       relation;
        RelationshipTarget target;

        while (eachStart.hasNext ()) {
            item = eachStart.next ();
            if (item instanceof Node) {
                padk = new PAssertionDataItem ((Node) item).getPAssertionDataKey ();
            } else {
                padk = (PAssertionDataKey) item;
            }
            keyStarts.add (padk);
            eachRelation = getRelationships (padk).iterator ();
            while (eachRelation.hasNext ()) {
                relation = (Relationship) eachRelation.next ();
                eachTarget = getRelationshipTargets (relation).iterator ();
                while (eachTarget.hasNext ()) {
                    target = (RelationshipTarget) eachTarget.next ();
                    if (_relationshipTargetCheck.accept (target)) {
                        results.add (new FullRelationship (padk, _storeURL, relation, target));
                        if (!excludes.contains (target)) {
                            excludes.add (target);
                            next.add (target);
                        }
                    }
                }
            }
        }
        if (!next.isEmpty ()) {
            results.addAll (resolve (next, excludes).getFullRelationships ());
        }
        
        return new ProvenanceQueryResults (keyStarts, results);
    }
}
