package org.pasoa.query.provenance;

import java.util.Iterator;
import org.pasoa.common.Constants;
import org.pasoa.common.structure.ObjectID;
import org.pasoa.common.structure.PAssertionDataKey;
import org.pasoa.common.structure.serialiser.PStructureSerialiser;
import org.pasoa.dommap.DOMSerialiser;
import org.pasoa.dommap.DOMWriter;
import org.w3c.dom.Document;

public class ProvenanceQuerySerialiser {
    public Document serialiseProvenanceQueryResults (ProvenanceQueryResults results) {
        DOMSerialiser build     = new DOMSerialiser ();
        Iterator      eachStart = results.getStarts ().iterator ();
        Iterator      eachRel   = results.getFullRelationships ().iterator ();
        
        build.namespace (Constants.PQUERY_NS, "pq");
        build.start ("provenanceQueryResult");
        build.start ("start");
        while (eachStart.hasNext ()) {
            build.append (serialisePAssertionDataKey ((PAssertionDataKey) eachStart.next ()));
        }
        build.end ();
        while (eachRel.hasNext ()) {
            build.append (serialiseFullRelationship ((FullRelationship) eachRel.next ()));
        }
        
        return (Document) build.getRoot ();
    }
    
    private Document serialisePAssertionDataKey (PAssertionDataKey pAssertionDataKey) {
        DOMSerialiser           build   = new DOMSerialiser ();
        PStructureSerialiser pstruct = new PStructureSerialiser ();
        
        build.namespace (Constants.PQUERY_NS, "pq");
        build.start ("pAssertionDataKey");
        build.namespace (Constants.PSTRUCT_NS, "ps");
        build.append (pstruct.serialiseInteractionKey (pAssertionDataKey.getGlobalPAssertionKey ().getInteractionKey ()));
        build.single ("viewKind", pAssertionDataKey.getGlobalPAssertionKey ().getViewName ());
        build.single ("localPAssertionId", pAssertionDataKey.getGlobalPAssertionKey ().getLocalPAssertionID ());
        build.append (pAssertionDataKey.getDataAccessor ());
        
        return (Document) build.getRoot ();
    }
    
    private Document serialiseFullRelationship (FullRelationship fullRelationship) {
        DOMSerialiser           build   = new DOMSerialiser ();
        PStructureSerialiser pstruct = new PStructureSerialiser ();
        
        build.namespace (Constants.PQUERY_NS, "pq");
        build.start ("fullRelationship");
        build.append (serialiseObjectID (Constants.PQUERY_NS, "fullSubjectId", fullRelationship.getSubject ()));
        build.single ("relation", fullRelationship.getRelation ());
        build.single ("localPAssertionId", fullRelationship.getLocalPAssertionID ());
        build.append (serialiseObjectID (Constants.PQUERY_NS, "fullObjectId", fullRelationship.getObject ()));
        
        return (Document) build.getRoot ();
    }
    
    private Document serialiseObjectID (String namespace, String elementName, ObjectID objectID) {
        DOMSerialiser        build   = new DOMSerialiser ();
        PStructureSerialiser pstruct = new PStructureSerialiser ();
        
        build.namespace (namespace);
        build.start (elementName);
        build.namespace (Constants.PSTRUCT_NS, "ps");
        build.append (pstruct.serialiseInteractionKey (objectID.getGlobalPAssertionKey ().getInteractionKey ()));
        build.single ("viewKind", objectID.getGlobalPAssertionKey ().getViewName ());
        build.single ("localPAssertionId", objectID.getGlobalPAssertionKey ().getLocalPAssertionID ());
        build.append (objectID.getDataAccessor ());
        build.single ("parameterName", objectID.getParameterName ());
        build.start ("objectLink");
        if (objectID.getLink () != null) {
            build.single ("provenanceStoreRef", objectID.getLink ().getStoreURL ());
        }
        
        return (Document) build.getRoot ();
    }
    
    public Document serialiseRelationshipTarget (RelationshipTarget relationshipTarget) {
        DOMSerialiser        build   = new DOMSerialiser ();
        PStructureSerialiser pstruct = new PStructureSerialiser ();
        
        build.namespace (Constants.PQUERY_NS, "pq");
        build.start ("relationshipTarget");
        build.namespace (Constants.PSTRUCT_NS, "ps");
        build.append (pstruct.serialiseInteractionKey (relationshipTarget.getGlobalPAssertionKey ().getInteractionKey ()));
        build.single ("viewKind", relationshipTarget.getGlobalPAssertionKey ().getViewName ());
        build.single ("localPAssertionId", relationshipTarget.getGlobalPAssertionKey ().getLocalPAssertionID ());
        build.copy   (relationshipTarget.getDataAccessor ());
        build.single ("parameterName", relationshipTarget.getParameterName ());
        build.start ("objectLink");
        if (relationshipTarget.getLink () != null) {
            build.single ("provenanceStoreRef", relationshipTarget.getLink ().getStoreURL ());
        }
        build.end ();
        build.single ("relation", relationshipTarget.getRelation ());
        build.copy (relationshipTarget.getAssertion ());
//System.out.println (DOMWriter.writeToString (build.getRoot (), true));
//System.out.println (build.getRoot ().getLastChild ().getLastChild ().getNamespaceURI ());
//System.out.println (build.getRoot ().getLastChild ().getLastChild ().getLocalName ());
//System.out.println (build.getRoot ().getLastChild ().getLastChild ().getParentNode ().getLocalName ());
        return (Document) build.getRoot ();
    }
}
