package org.pasoa.query.xpath;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.pasoa.common.Constants;
import org.pasoa.dommap.DOMDeserialiser;
import org.pasoa.dommap.DOMSerialiser;
import org.pasoa.query.QueryProblem;
import org.pasoa.query.xpath.jaxen.XPathUtilities;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class SingleNodeXPath {
    private String _path;
    private Map    _namespaces;
    
    public SingleNodeXPath (String path, Map namespaces) {
        _namespaces = namespaces;
        _path       = path;
    }
    
    public SingleNodeXPath (SingleNodeXPath head, SingleNodeXPath tail) {
        this (head._path + tail._path, merged (head._namespaces, tail._namespaces));
    }
    
    public SingleNodeXPath addNamespace (String prefix, String namespace) {
        _namespaces.put (prefix, namespace);
        return this;
    }
    
    public boolean equals (Object other) {
        if (other instanceof SingleNodeXPath) {
            return ((SingleNodeXPath) other).expandedForm ().equals (expandedForm ());
        } else {
            return false;
        }
    }
    
    public String expandedForm () {
        String expanded = _path;
        int    start    = 0;
        int    colon;
        int    slash;
        String prefix;
        String namespace;
        
        while (start >= 0 && expanded.indexOf (":", start) >= 0) {
            colon     = expanded.indexOf (":", start);
            slash     = expanded.lastIndexOf ("/", colon);
            prefix    = expanded.substring (slash + 1, colon); 
            namespace = (String) _namespaces.get (prefix);
            expanded  = expanded.substring (0, slash + 1) + " " + namespace + " " + expanded.substring (colon + 1);
            start     = expanded.indexOf ("/", slash + namespace.length () + 3);
        }
        
        return expanded;
    }

    public static SingleNodeXPath fromXML (Element serialised) {
        DOMDeserialiser parse  = new DOMDeserialiser (Constants.PQUERY_NS);
        String          path   = parse.text (serialised, "path");
        Iterator        eachNS = parse.iterator (serialised, "namespaceMapping");
        Map             spaces = new HashMap ();
        Element         map;
        
        while (eachNS.hasNext ()) {
            map = (Element) eachNS.next ();
            spaces.put (parse.text (map, "prefix"), parse.text (map, "namespace"));
        }
        
        return new SingleNodeXPath (path, spaces);
    }
    
    public static SingleNodeXPath generate (Node node, Node root) {
        Map             namespaces = new HashMap ();
        Node            parent     = node.getParentNode ();
        int             index      = 0;
        SingleNodeXPath part;
        
        if (!(root != null && node.isSameNode (root)) && parent != null && !(parent instanceof Document)) {
            index = indexOfChild (parent, node);
        }
        
        if (node instanceof Element) {
            namespaces.put (node.getPrefix (), node.getNamespaceURI ());
            part = new SingleNodeXPath ("/" + node.getPrefix () + ":" + node.getLocalName () + "[0]", namespaces);
        } else {
            part = new SingleNodeXPath ("/text()[" + index + "]", namespaces);
        }
        
        if (!(root != null && node.isSameNode (root)) && parent != null && !(parent instanceof Document)) {
            return new SingleNodeXPath (generate (parent, root), part);
        } else {
            return part;
        }
    }
    
    public static SingleNodeXPath generate (Node node) {
        return generate (node, null);
    }
    
    public int hashCode () {
        return expandedForm ().hashCode ();
    }
    
    public static int indexOfChild (Node parent, Node child) {
        NodeList siblings = parent.getChildNodes ();
        
        for (int index = 0; index < siblings.getLength (); index += 1) {
            if (siblings.item (index).isSameNode (child)) {
                return index;
            }
        }
        
        return -1;
    }
    
    private static Map merged (Map first, Map second) {
        Map      merged  = new HashMap (first);
        Iterator eachKey = second.keySet ().iterator ();
        Object   key;
        
        while (eachKey.hasNext ()) {
            key = eachKey.next ();
            merged.put (key, second.get (key));
        }
        
        return merged;
    }
    
    public Node resolve (Element document) throws QueryProblem {
        return (Node) XPathUtilities.resolveNodeSet (document, _path, _namespaces).get (0);
    }
    
    public Element toXML () {
        DOMSerialiser writer     = new DOMSerialiser ();
        Iterator      eachPrefix = _namespaces.keySet ().iterator ();
        String        prefix;
        
        writer.namespace (Constants.PQUERY_NS);
        writer.start ("singleNodeXPath");
        writer.single ("path", _path);
        while (eachPrefix.hasNext ()) {
            prefix = (String) eachPrefix.next ();
            writer.start ("namespaceMapping");
            writer.single ("prefix",    prefix);
            writer.single ("namespace", (String) _namespaces.get (prefix));
            writer.end ();
        }
        
        return ((Document) writer.getRoot ()).getDocumentElement ();
    }
}
