package org.pasoa.query.xpath.jaxen;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.jaxen.BaseXPath;
import org.jaxen.JaxenException;
import org.jaxen.SimpleNamespaceContext;
import org.jaxen.XPathFunctionContext;
import org.jaxen.dom.DocumentNavigator;
import org.pasoa.common.Constants;
import org.pasoa.dommap.DOMWriter;
import org.pasoa.dommap.SimpleText;
import org.pasoa.query.QueryProblem;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * A set of utilities for extracting values from DOM documents using XPath.
 */
public class XPathUtilities {
    private XPathUtilities () {
    }
    
    public static Map resolveMap (Element document, String mapsPath, String keyPath, String valuePath, Map namespaces) throws QueryProblem {
        Map      results = new HashMap ();
        Iterator eachMap = resolveNodeSet (document, mapsPath, namespaces).iterator ();
        Element  mapNode;
        
        while (eachMap.hasNext ()) {
            mapNode = (Element) eachMap.next ();
            results.put (resolveString (mapNode, keyPath, namespaces), resolveString (mapNode, valuePath, namespaces));
        }
        
        return results;
    }

    private static Object resolve (Element document, String xpath, Map namespaces) throws QueryProblem {
        try {
            BaseXPath            encoded   = new BaseXPath (xpath, new DocumentNavigator ());
            XPathFunctionContext functions = new XPathFunctionContext ();
            
//System.out.println ("XPath: " + xpath);            
//System.out.println ("Namespaces: " + namespaces);
//System.out.println ("Document: " + DOMWriter.writeToString (document));
            //functions.registerFunction (Constants.PQUERY_NS, "isSubclassOf", new SubclassOfFunction ());
            encoded.setFunctionContext (functions);
            encoded.setNamespaceContext (new SimpleNamespaceContext (namespaces));
            
            return encoded.evaluate (document);
        } catch (Exception xpathProblem) {
            xpathProblem.printStackTrace ();
            throw new QueryProblem (xpathProblem);
        }
    }
    
    /**
     * In the XPath specification and here, a node set includes a set of values as well as
     * a set of nodes.
     */
    public static List resolveNodeSet (Element document, String xpath, Map namespaces) throws QueryProblem {
        return (List) resolve (document, xpath, namespaces);
    }
    
    public static String resolveString (Element document, String xpath, Map namespaces) throws QueryProblem {
        return resolve (document, xpath, namespaces).toString ();
    }
    
    public static void setNodeText (Element document, String xpath, String newValue, Map namespaces) throws QueryProblem {
        Iterator eachNode = resolveNodeSet (document, xpath, namespaces).iterator ();
        boolean  checked;
        Node     node;
        NodeList children;

        while (eachNode.hasNext ()) {
            node    = (Node) eachNode.next ();
            checked = true;
            do {
                children = node.getChildNodes ();
                for (int index = 0; index < children.getLength (); index += 1) {
                    if (children.item (index).getNodeType () == Node.TEXT_NODE) {
                        node.removeChild (children.item (index));
                        checked = false;
                        break;
                    }
                }
            } while (!checked);
            node.appendChild (new SimpleText (document.getOwnerDocument (), newValue));
        }
    }

    public static boolean resolveBoolean (Element document, String xpath, Map namespaces) throws QueryProblem {
        return ((Boolean) resolve (document, xpath, namespaces)).booleanValue ();
    }
}
