package org.pasoa.query.xquery;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import javax.xml.namespace.QName;
import org.pasoa.common.Constants;
import org.pasoa.dommap.SimpleDocument;
import org.pasoa.dommap.SimpleElement;
import org.pasoa.query.QueryProblem;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class XQueryProcessor {
    public static Collection resolveXPath (String path, Map namespaces, Node storeDoc, XQueryResolver engine) throws QueryProblem {
        String     xquery     = "$ps:pstruct" + path;
        Iterator   eachPrefix = namespaces.keySet ().iterator ();
        Map        documents  = new HashMap ();
        Map        variables  = new HashMap ();
        String     prefix;
        String     namespace;
        Object     answer;
        Collection results;
        
        while (eachPrefix.hasNext ()) {
            prefix    = (String) eachPrefix.next ();
            namespace = (String) namespaces.get (prefix);
            xquery = "declare namespace " + prefix + " = \"" + namespace + "\"; " + xquery;
        }
        variables.put (new QName (Constants.PSTRUCT_NS, "pstruct", "ps"), ((Document) storeDoc).getDocumentElement ());

        answer = engine.resolve (xquery, documents, variables);
        
        if (answer instanceof Document) {
            answer = ((Document) answer).getDocumentElement ();
        }
        if (answer instanceof Element) {
            results = new LinkedList ();
            results.add (answer);
            return results;
        }
        return (Collection) answer;
    }
    
    public static Document resolve (String query, Document storeDoc, XQueryResolver engine) throws Exception {
        Map      spaces    = new HashMap ();
        Map      documents = new HashMap ();
        Map      variables = new HashMap ();
        String   pasoaNS   = "declare namespace " + "ps" + " = \"" + Constants.PSTRUCT_NS + "\"; ";
        String   storeVar  = "declare variable $" + "ps" + ":pstruct external ;";
        String   newProlog = pasoaNS + storeVar;
        Document result;
        Object   answer;
        
        query = newProlog + query;
        spaces.put ("ps", Constants.PSTRUCT_NS);
        spaces.put (Constants.WSXQUERY_PREFIX, Constants.WSXQUERY_NS);
        
        try {
            documents.put ("store", storeDoc);
            variables.put (new QName (Constants.PSTRUCT_NS, "pstruct", "ps"), storeDoc.getDocumentElement ());
            System.out.println ("Before the resolve...");
            answer = engine.resolve (query, documents, variables);
            System.out.println ("After the resolve");
            if (answer instanceof Document) {
                result = new SimpleDocument (spaces);
                SimpleElement se = new SimpleElement (result, Constants.WSXQUERY_NS, "queryAck");
                result.appendChild (se);
                se.appendChild (((Document) answer).getDocumentElement());
                return result;
            }
            if (answer instanceof Element) {
                result = new SimpleDocument (spaces);
                SimpleElement se = new SimpleElement (result, Constants.WSXQUERY_NS, "queryAck");
                result.appendChild (se);
                se.appendChild ((Element) answer);
                return result;
            }
            throw new QueryProblem (new Exception ("Unrecognised XQuery response type: " + answer.getClass ()));
        } catch (Exception storeProblem) {
            throw new QueryProblem (storeProblem);
        }
    }
}
