package org.pasoa.query.xquery.ipsixq;

import de.fraunhofer.ipsi.ipsixq.api.impl.IPSIXQDataSource;
import java.util.Iterator;
import java.util.Map;
import javax.xml.namespace.QName;
import javax.xml.xquery.XQConnection;
import javax.xml.xquery.XQDatasource;
import javax.xml.xquery.XQException;
import javax.xml.xquery.XQExpression;
import javax.xml.xquery.XQItem;
import javax.xml.xquery.XQResultSequence;
import org.pasoa.query.QueryProblem;
import org.pasoa.query.xquery.XQueryResolver;
import org.w3c.dom.Node;

public class XQueryUtilities implements XQueryResolver {
    public Object resolve (String query, Map documents, Map variables) throws QueryProblem {
        try {
            return resolveSingle (query, documents, variables).getNode ();
        } catch (XQException ipsiProblem) {
            throw new QueryProblem (ipsiProblem);
        }
    }
    
    public boolean resolveContainsAny (String query, Map documents, Map variables) throws QueryProblem {
        try {
            return resolveSequence (query, documents, variables).isAfterLast ();
        } catch (XQException ipsiProblem) {
            throw new QueryProblem (ipsiProblem);
        }
    }
    
    public XQResultSequence resolveSequence (String query, Map documents, Map variables) throws QueryProblem {
        System.out.println ("IPSIXQ: got a call");
        try {
            //establish connection to IPSI-XQ engine
            XQDatasource     dataSource = new IPSIXQDataSource ();
            XQConnection     connection = dataSource.getConnection ();
            Iterator         eachVar    = variables.keySet ().iterator ();
            QName            varName;
            Object           varValue;
            XQExpression     expression;
            XQResultSequence sequence;
            
            while (eachVar.hasNext ()) {
                varName  = (QName) eachVar.next ();
                varValue = variables.get (varName);
                if (varValue instanceof Node) {
                    connection.bindNode (varName, (Node) varValue);
                }
                if (varValue instanceof String) {
                    connection.bindString (varName, (String) varValue);
                }
                // ...etc.
            }
            
            //create the expression used to carry out the query
            expression = connection.createExpression ();
            System.out.println (query);
            //execute query
            
            return expression.executeQuery (query);
        } catch (Exception ipsiProblem) {
            throw new QueryProblem (ipsiProblem);
        }
    }
    
    public XQItem resolveSingle (String query, Map documents, Map variables) throws QueryProblem {
        XQResultSequence sequence = resolveSequence (query, documents, variables);
        
        try {
            sequence.next ();
            return sequence.getItem ();
        } catch (XQException ipsiProblem) {
            throw new QueryProblem (ipsiProblem);
        }
    }
}
