package org.pasoa.record;

import java.io.StringReader;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.pasoa.common.Constants;

import org.pasoa.common.PlugIn;
import org.pasoa.common.structure.GlobalPAssertionKey;
import org.pasoa.common.structure.InteractionKey;
import org.pasoa.common.structure.deserialiser.PStructureDeserialiser;
import org.pasoa.dommap.DOMWriter;
import org.pasoa.storage.StorageSystem;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class RecordPlugIn implements PlugIn {
    public  static final String NO_LPID      = "nolocalpassertionid";
    
    private boolean _sync;
    private DocumentBuilderFactory _dbf;
    
    public RecordPlugIn () {
        _dbf = DocumentBuilderFactory.newInstance ();
        _dbf.setNamespaceAware (true);
        
        // Default is to receive dispatches asynchrounsly
        _sync = false;
    }
        
    public Document process (Document document, StorageSystem ss)
    throws Exception {
        StringBuffer ackStr = new StringBuffer ("<pr:recordAck xmlns:pr=\"" + Constants.PRECORD_NS + "\">");
        
        
        NodeList nl = document.getDocumentElement ().getChildNodes ();
        Element soapBody = null;
        for (int i = 0; i < nl.getLength (); i++) {
            if (nl.item (i) instanceof Element) {
                if (((Element) nl.item (i)).getTagName ().endsWith ("Body")) {
                    soapBody = (Element) nl.item (i);
                    break;
                }
            }
        }
        
        NodeList idContents = soapBody.getElementsByTagNameNS (Constants.PRECORD_NS, "identifiedContent");
        
        for (int i = 0; i < idContents.getLength (); i++) {
            Element idc = (Element) idContents.item (i);
            
            Element identifiers = (Element) idc.getElementsByTagNameNS (Constants.PSTRUCT_NS, "interactionKey").item (0);
            
            String interactionId = identifiers.getElementsByTagNameNS (Constants.PSTRUCT_NS, "interactionId").item (0).getChildNodes ().item (0).getNodeValue ();
            
            Element sink = PStructureDeserialiser.getFirstChildElement ((Element) identifiers.getElementsByTagNameNS (Constants.PSTRUCT_NS, "messageSink").item (0));

            Element source = PStructureDeserialiser.getFirstChildElement ((Element) identifiers.getElementsByTagNameNS (Constants.PSTRUCT_NS, "messageSource").item (0));
            
            InteractionKey irid = new InteractionKey (source, sink, interactionId);
            
            Element asserter = PStructureDeserialiser.getFirstChildElement ((Element)idc.getElementsByTagNameNS (Constants.PSTRUCT_NS, "asserter").item (0));
            
            String viewKind = idc.getElementsByTagNameNS (Constants.PSTRUCT_NS, "viewKind").item (0).getChildNodes ().item (0).getNodeValue ();
            
            
            
            NodeList contents = idc.getElementsByTagNameNS (Constants.PRECORD_NS, "content");
            
            //System.out.println("Number of contents " + contents.getLength());
            for (int j = 0; j < contents.getLength (); j++) {
                Element content = (Element) contents.item (j);
                
                String contentName = content.getElementsByTagName ("*").item (0).getLocalName ();
                //System.out.println (contentName);
                
                String localPAssertionId;
                if (contentName.equalsIgnoreCase ("submissionFinished")) {
                    localPAssertionId = NO_LPID;
                } else {
                    localPAssertionId = content.getElementsByTagNameNS (Constants.PSTRUCT_NS, "localPAssertionId").item (0).getChildNodes ().item (0).getNodeValue ();
                }
                //System.out.println ("LPID " + localPAssertionId);
                
                
                
                GlobalPAssertionKey gpid = new GlobalPAssertionKey (irid, viewKind, localPAssertionId);
                
                // Record the assertion in the underlying storage system
                ss.record (gpid, asserter, PStructureDeserialiser.getFirstChildElement (content), contentName);
                
                
                // Make the acknowledgment document
                
                ackStr.append ("<pr:ack xmlns:pr=\"" + Constants.PRECORD_NS + "\" xmlns:ps=\"" + Constants.PSTRUCT_NS + "\">");
                ackStr.append ("<pr:contentName>");
                ackStr.append (contentName);
                ackStr.append ("</pr:contentName>");
                ackStr.append ("<ps:interactionKey>");
                
                ackStr.append ("<ps:messageSource>");
                ackStr.append (DOMWriter.writeToString (irid.getMessageSource ()));
                ackStr.append ("</ps:messageSource>");
                
                ackStr.append ("<ps:messageSink>");
                ackStr.append (DOMWriter.writeToString (irid.getMessageSink ()));
                ackStr.append ("</ps:messageSink>");
                
                ackStr.append ("<pr:interactionId>");
                ackStr.append (interactionId);
                ackStr.append ("</pr:interactionId>");
                
                
                ackStr.append ("</ps:interactionKey>");
                ackStr.append ("</pr:ack>");
            }
            
        }
        ackStr.append ("</pr:recordAck>");
        
        DocumentBuilder docBuilder = _dbf.newDocumentBuilder ();
        Document retDoc = docBuilder.parse (new InputSource (new StringReader (ackStr.toString ())));
        
        return retDoc;
    }
    
    public void setReceiveDipatchesSynchronously (boolean sync) {
        _sync = sync;
    }
    
    public boolean receiveDispatchesSyncronously () {
        return _sync;
    }
    
    public String getDefaultResponse () {
        return "<pr:recordAck xmlns:pr=\"" + Constants.PRECORD_NS + "\"><pr:synch_ack/></pr:recordAck>";
    }

    public Map getConfiguration()
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void initialize(Map configuration)
    {
        // TODO Auto-generated method stub
        
    }
    
}
