package org.pasoa.servlet;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.util.Enumeration;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.pasoa.common.Constants;
import org.pasoa.dommap.DOMWriter;
import org.pasoa.storage.BackendStore;
import org.pasoa.factory.*;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class ProvenanceService extends HttpServlet
{
    private BackendStore            _backend;
    private String                  _storagePath;
    private long                    _cacheWaitTime;
    private ProvenanceStoreRegistry _pStoreRegistry;
    
    public void init() throws ServletException
    {
        
        
        System.out.println ("initialize");
        
        _pStoreRegistry = new ProvenanceStoreRegistry();
        
        _storagePath = getInitParameter ("StorePath");
        System.out.println("PReServ: Storage path is: " + _storagePath);
        
        try {
            System.out.println ("PReServ: Getting cache wait time");
            String cacheWaitStr = getInitParameter ("CacheWaitTime");
            long cacheTime = Long.parseLong(cacheWaitStr);
            _cacheWaitTime = cacheTime;
        } catch (Exception e) {
            System.out.println ("Cache wait time not set, using default");
            _cacheWaitTime = 75;
        }
        
        /*String backendKind = getInitParameter ("Backend");
        
        if (backendKind.equalsIgnoreCase("InMemory")) {
            _backend = new MemoryStorage ();
        } 
        else if (backendKind.equalsIgnoreCase("BerkeleyJDB")) {
            _backend = new BerkeleyDBJEStorage (_storagePath);
        }
        _storageSystem = new GeneralStorageSystem (_backend, "org.pasoa.query.xquery.ipsixq.XQueryUtilities");
  
        */
        
        try {
             ProvenanceStore pStore = ProvenanceStoreFactory.newInstance(_storagePath + "/default", _cacheWaitTime);
        
             _pStoreRegistry.registerPlugIn("/default", pStore);
        } catch (Exception regException) {
            System.out.println ("PReServ: Could not create or register default provenance store");
            regException.printStackTrace();
        }
        
        System.out.println ("PReServ: Done with initialization");
        
    }
    
    public void doGet (HttpServletRequest request, HttpServletResponse response) 
    {
        try {
        String scheme = request.getScheme();             // http
        String serverName = request.getServerName();     // hostname.com
        int serverPort = request.getServerPort();        // 80
        String contextPath = request.getContextPath();   // /mywebapp
        String servletPath = request.getServletPath();   // /servlet/MyServlet
        String queryStr = request.getQueryString();
        String pathInfo = request.getPathInfo();
        // Reconstruct original requesting URL
        String servletURL = scheme+"://"+serverName+":"+serverPort+contextPath;
       
        System.out.println (servletPath);
        System.out.println (servletURL);
        System.out.println (pathInfo);
        
        System.out.println ("PReServ: Loading Management Page");
        response.setStatus (HttpServletResponse.SC_OK);
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter pw = response.getWriter();
        pw.write("<html>");
        pw.write("<body>");
        pw.write("<img src=\"" + servletURL + "/PReServLogo.gif\"/>");
        pw.write("<br>");
        /*
        if (_backend instanceof BerkeleyDBJEStorage) {
            BerkeleyDBJEStorage jedb = (BerkeleyDBJEStorage)_backend;
            
            if (pathInfo.equals("/")) {
            
                long numIRs = jedb.getNumberOfInteractionRecords();
                for (long i = 0; i < numIRs; i++) {
                    InteractionRecord ir = jedb.getInteractionRecord(i);
                    StringBuffer id = new StringBuffer();
                    id.append("&");
                    id.append(ir.getInteractionRecordID().getInteractionID());
                    id.append ("&");
                    id.append(ir.getInteractionRecordID().getClientID());
                    id.append ("&");
                    id.append(ir.getInteractionRecordID().getMessageSource());
                    id.append ("&");
                    id.append(ir.getInteractionRecordID().getServiceID());
                    id.append ("&");
                    id.append(ir.getInteractionRecordID().getMessageSink());
                    String urledId = URLEncoder.encode(id.toString(), "UTF-8");
                    pw.write("<a href=\"" + servletURL + servletPath  + "/" + urledId +"\">" + ir.getInteractionRecordID().getInteractionID() + "</a>");
                    pw.write("<br>");
                }
            }
            else {
                try {
                    String results[] = pathInfo.split("&");
                    for (int r = 0; r < results.length; r++) {
                         System.out.println (results[r]);    
                    }
                    InteractionRecordID findRec = new InteractionRecordID (results[3], results[4], results[5], results[6], results[2]);
                
                    InteractionRecord nir = jedb.getInteractionRecord(findRec);
                    nir.getInteractionRecordID();
                    pw.write("found interaction record");
                } catch (Exception nf) {
                    pw.write ("No records found");
                }
            }
            
        }
        */
        pw.write("</body>");
        pw.write("</html>");
        pw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
    }
    
    /*private void processManageSetString (String queryStr) throws Exception
    {
        // Example query string
        // backend=berkeleydb&storepath=%2FUsers%2Fpgroth%2FDevelop%2FStore&cachepausetime=1000
        String decodedQueryStr = URLDecoder.decode (queryStr, "UTF-8");
        System.out.println("Decoded String: " + decodedQueryStr);
        // Example query string after decoding
        // backend=berkeleydb&storepath=/Users/pgroth/Develop/store&cachepausetime=200
        StringTokenizer st = new StringTokenizer (decodedQueryStr, "&");
        
        // Get the backend store
        String backend = st.nextToken();
        int index = backend.indexOf("=");
        backend = backend.substring(index+1);
        System.out.println (backend);
        
        // Get the store path
        String storePath = st.nextToken();
        index = storePath.indexOf("=");
        storePath = storePath.substring(index+1);
        System.out.println (storePath);
        
        // Get the store path
        String cachePauseTime = st.nextToken();
        index = cachePauseTime.indexOf("=");
        cachePauseTime = cachePauseTime.substring(index+1);
        System.out.println (cachePauseTime);
        
        String oldStoragePath = _storagePath;
        if (!(storePath.equals(""))) {
            _storagePath = storePath;
        }
        
        if (backend.equals ("memory")) {
            if (!(_backend instanceof MemoryStorage)) {
                _backend = new MemoryStorage();
            }
        }
        else if (backend.equals ("berkeleydb")) {
            if (!(_backend instanceof BerkeleyDBJEStorage)) {
                _backend = new BerkeleyDBJEStorage (_storagePath);
            } 
            else if (!(_storagePath.equals (oldStoragePath))) {
                
            }
        }
        
    }*/
    
   /* private void generateManagementHTML()
    {
        if (servletPath.equalsIgnoreCase("/management")) {
            
            if (queryStr != null) {
                processManageSetString (queryStr);
            }
            
            System.out.println ("PReServ: Loading Management Page");
            response.setStatus (HttpServletResponse.SC_OK);
            response.setContentType("text/html;charset=UTF-8");
            PrintWriter pw = response.getWriter();
            pw.write("<html>");
            pw.write("<body>");
            pw.write("<img src=\"" + servletURL + "/PReServLogo.gif\"/>");
            pw.write("<br>");
            pw.write("<b>Manage</b>");
            String action = "management";
            pw.write("<FORM name=\"input\" action=\"" + action + "\" method=\"get\">");
            pw.write("<P>");
            
            if (_backend instanceof MemoryStorage) {
                pw.write("<input type=\"radio\" checked name=\"backend\" value=\"memory\"> In-Memory Store");
            }
            else {
                pw.write("<input type=\"radio\" name=\"backend\" value=\"memory\"> In-Memory Store");
            }
            
            pw.write("<br>");
            
            if (_backend instanceof BerkeleyDBJEStorage) {
                pw.write("<input type=\"radio\" checked name=\"backend\" value=\"berkeleydb\">Java Object Database");
            } 
            else {
                pw.write("<input type=\"radio\" name=\"backend\" value=\"berkeleydb\">Java Object Database");
            }
            
            pw.write("<br>");
            pw.write("<LABEL for=\"path\">Store Path: </LABEL>");
            pw.write("<INPUT type=\"text\" name=\"storepath\" value=\"" + _storagePath  + "\"><BR>");
            pw.write("<br>");
            pw.write("<LABEL for=\"processTime\">Pause time before processing cached items (in milliseconds): </LABEL>");
            pw.write("<INPUT type=\"text\" name=\"cachepausetime\" value=\"" + _dispatcher.getProcessCacheItemsWaitTime() + "\"><BR>");
            pw.write("<INPUT type=\"submit\" value=\"Submit\">");
            pw.write("</P></FORM>");
            pw.write("</body>");
            pw.write("</html>");
            pw.close();
            
            // Things to set
            // - which backend
            // - path for preserv to put stuff at
            // - available plug ins
            // - whether a plug in uses caching
            // - time when the thread runs to process cached items
            
        }
    }*/
    
    public void doPost (HttpServletRequest request, HttpServletResponse response) throws ServletException
    {

        //System.out.println ("Post");
       
        try {
           String servlet = request.getServletPath();
           System.out.println ("PReServ: Servlet " + servlet);
           
           String store = request.getPathInfo();
           System.out.println ("PReServ: Store " + store);
           
           if (store == null) {
               System.out.println ("PReServ: No store given");
               System.out.println ("PReServ: Using default store");
               store = "/default";
           }
           
           response.setStatus (HttpServletResponse.SC_OK);
           response.setContentType ("application/soap+xml; charset=utf-8");
           
           PrintWriter pw = new PrintWriter (response.getOutputStream());
           pw.print ("<env:Envelope xmlns:env=\"http://schemas.xmlsoap.org/soap/envelope/\">");
           pw.print ("<env:Header>");
           pw.print ("</env:Header>");
           pw.print ("<env:Body>");
           pw.flush();
           
           if (servlet.equalsIgnoreCase("/manage")) {
               System.out.println ("Handle Management");
           }
           else {
               ProvenanceStore pStore = _pStoreRegistry.lookup(store);
               pStore.getDispatcher().processAndDispatch(request.getInputStream(), response.getOutputStream(), servlet);
               
           }
           
           pw.print ("</env:Body>");
           pw.print ("</env:Envelope>");
           pw.close();
           System.out.println("Replied");
           
            
           /*
           Enumeration attributes = request.getHeaderNames();
           while (attributes.hasMoreElements()) {
               String headerName = (String)attributes.nextElement();
               System.out.println(headerName);
               System.out.println(request.getHeader(headerName));
           }
           
           System.out.println ("user " + request.getRemoteUser());
           System.out.println("AuthType " + request.getAuthType());
           
           System.out.println ("protocol " + request.getProtocol());
           
           if (request.isSecure()) {
               System.out.println ("Request is secure");
           }
           
           String cipherSuite = (String) request.getAttribute("javax.servlet.request.cipher_suite");
           System.out.println("Cipher Suite: " + cipherSuite);
           if (cipherSuite != null) {
                //System.out.println(request.getAttribute("org.apache.coyote.request.X509Certificate"));
           }
           */
             
           
           
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServletException (e);
        }
    }
    
    public void handleManagement (InputStream is, OutputStream os) throws Exception
    {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        DocumentBuilder docBuilder = dbf.newDocumentBuilder();
        Document doc = docBuilder.parse(is);
        
        StringBuffer ackStrBuf = new StringBuffer();
       
        NodeList nl = doc.getDocumentElement().getElementsByTagNameNS(Constants.PMANAGE_NS, "createRepository");
        boolean createRepository = false;
        if (nl.getLength() > 0) {
            createRepository = true;
        }
        nl = doc.getDocumentElement().getElementsByTagNameNS(Constants.PMANAGE_NS, "deleteRepository");
        boolean deleteRepository = false;
        if (nl.getLength() > 0) {
            deleteRepository = true;
        }
        
        nl = doc.getDocumentElement().getElementsByTagNameNS(Constants.PMANAGE_NS, "listRepositories");
        boolean list = false;
        if (nl.getLength() > 0) {
            list = true;
        }
        
        
        if (createRepository) {
            boolean created = true;
            Node createRepositoryNode = nl.item(0);
            String repositoryName = createRepositoryNode.getChildNodes ().item (0).getNodeValue ();
            if (repositoryName.indexOf("/") == -1) {
                created = false;
            } else {
                ProvenanceStore pStore = ProvenanceStoreFactory.newInstance(_storagePath + "/" + repositoryName, _cacheWaitTime);
                _pStoreRegistry.registerPlugIn("/" + repositoryName, pStore);
                   
            }
            ackStrBuf.append("<pm:createRepositoryAck xmlns:pm=\"" + Constants.PMANAGE_NS + "\">");
            ackStrBuf.append("<pm:repository>");
            ackStrBuf.append("<pm:repositoryName>");
            ackStrBuf.append(repositoryName);
            ackStrBuf.append("</pm:repositoryName>");
            ackStrBuf.append("</pm:repository>");
            ackStrBuf.append("<pm:created>");
            ackStrBuf.append(created);
            ackStrBuf.append("</pm:created>");
            ackStrBuf.append("</pm:createRepositoryAck>");
        }
        
        
        if (deleteRepository) {
            boolean deleted = true;
            Node deleteRepositoryNode = nl.item(0);
            String repositoryName = deleteRepositoryNode.getChildNodes ().item (0).getNodeValue ();
            if (repositoryName.indexOf("/") == -1) {
                deleted = false;
            } else {
                ProvenanceStore pStore = _pStoreRegistry.lookup("/" + repositoryName);
                _pStoreRegistry.deregisterPlugIn("/" + repositoryName);
                pStore.delete();
                   
            }
            ackStrBuf.append("<pm:deleteRepositoryAck xmlns:pm=\"" + Constants.PMANAGE_NS + "\">");
            ackStrBuf.append("<pm:repository>");
            ackStrBuf.append("<pm:repositoryName>");
            ackStrBuf.append(repositoryName);
            ackStrBuf.append("</pm:repositoryName>");
            ackStrBuf.append("</pm:repository>");
            ackStrBuf.append("<pm:deleted>");
            ackStrBuf.append(deleted);
            ackStrBuf.append("</pm:deleted>");
            ackStrBuf.append("</pm:deleteRepositoryAck>");
        }
        
        if (list) {
            
            ackStrBuf.append("<pm:listRepositoriesAck xmlns:pm=\"" + Constants.PMANAGE_NS + "\">");
            
            Enumeration names = _pStoreRegistry.storeNames();
            
            while (names.hasMoreElements()) {
                ackStrBuf.append("<pm:repository>");
                ackStrBuf.append("<pm:repositoryName>");
                ackStrBuf.append(names.nextElement());
                ackStrBuf.append("</pm:repositoryName>");
                ackStrBuf.append("</pm:repository>");
            }
            
            ackStrBuf.append("</pm:listRepositoriesAck>");
        }

        

        Document responseDoc = docBuilder.parse (new InputSource (new StringReader (ackStrBuf.toString ())));
        DOMWriter domWriter = new DOMWriter(os);
        domWriter.write(responseDoc);
   
    }
    
    public void destroy()
    {
        try {
            System.out.println ("Calling dispatcher shutdown");
            
            _pStoreRegistry.shutdownAll();

        } catch (Exception e) {
            System.out.println ("PReServ: Problems during shutdown");
            e.printStackTrace();
        }
    }
     
    
}
