package org.pasoa.storage;

import java.util.HashMap;
import java.util.Map;
import org.pasoa.common.CacheableItem;
import org.pasoa.common.Constants;
import org.pasoa.common.Retrieval;
import org.pasoa.common.structure.GlobalPAssertionKey;
import org.pasoa.query.QueryProblem;
import org.pasoa.query.dom.QStructureDocument;
import org.pasoa.query.xquery.XQueryProcessor;
import org.pasoa.query.xquery.XQueryResolver;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class GeneralStorageSystem implements StorageSystem {
    
    private BackendStore   _store;
    private XQueryResolver _engine;
    
    public GeneralStorageSystem (BackendStore store, String resolverClass) {
        _store = store;
        try {
            _engine = (XQueryResolver) Class.forName (resolverClass).newInstance ();
            
            if (_engine instanceof org.pasoa.query.xquery.ipsixq.XQueryUtilities) {
                System.out.println ("Engine is ipsixq");
            } else {
                System.out.println ("ipsixq not loaded");
            }
        } catch (Exception problem) {
            throw new RuntimeException ("Fatal error: " + problem);
        }
    }
    
    public void record (GlobalPAssertionKey gpid, Element asserter, Element assertion,
            String kindOfAssertion) throws Exception {
        _store.record (gpid, asserter, assertion, kindOfAssertion);
    }
    
    public Document resolve (String query) throws Exception {
        try {
            Map spaces = new HashMap ();
            
            spaces.put ("ps", Constants.PSTRUCT_NS);
            spaces.put (Constants.WSXQUERY_PREFIX, Constants.WSXQUERY_NS);
            
            return XQueryProcessor.resolve (query, new QStructureDocument (_store, spaces), _engine);
        } catch (Exception storeProblem) {
            throw new QueryProblem (storeProblem);
        }
    }
    
    public boolean cacheEnabled () {
        return _store.cacheEnabled ();
    }
    
    public void cache (CacheableItem ci) throws Exception {
        _store.cache (ci);
    }
    
    public CacheableItem getCachedItem () throws Exception {
        return _store.getCachedItem ();
    }
    
    public Retrieval getRetrievalInterface () {
        return _store;
    }
    
    public void shutdown () throws Exception {
        _store.shutdown ();
    }
    
    public void delete() throws Exception {
        _store.delete();
    }
}
