package org.pasoa.storage.je;

import java.io.File;
import java.util.LinkedList;

import org.pasoa.common.CacheableItem;
import org.pasoa.common.Constants;
import org.pasoa.common.structure.ActorStatePAssertion;
import org.pasoa.common.structure.GlobalPAssertionKey;
import org.pasoa.common.structure.InteractionPAssertion;
import org.pasoa.common.structure.InteractionRecord;
import org.pasoa.common.structure.InteractionKey;
import org.pasoa.common.structure.Relationship;
import org.pasoa.common.structure.View;
import org.pasoa.common.structure.deserialiser.PStructureDeserialiser;
import org.pasoa.storage.BackendStore;
import org.pasoa.storage.memory.MemoryStorage;
import org.w3c.dom.Element;

import com.sleepycat.bind.EntryBinding;
import com.sleepycat.bind.serial.SerialBinding;
import com.sleepycat.bind.serial.StoredClassCatalog;
import com.sleepycat.bind.tuple.TupleBinding;
import com.sleepycat.je.Cursor;
import com.sleepycat.je.Database;
import com.sleepycat.je.DatabaseConfig;
import com.sleepycat.je.DatabaseEntry;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.DeadlockException;
import com.sleepycat.je.Environment;
import com.sleepycat.je.EnvironmentConfig;
import com.sleepycat.je.LockMode;
import com.sleepycat.je.OperationStatus;
import com.sleepycat.je.SecondaryConfig;
import com.sleepycat.je.SecondaryDatabase;
import com.sleepycat.je.Transaction;

public class BerkeleyDBJEStorage implements BackendStore {
    public static final String TOTAL_NUM_RECORDS_KEY = "totalnumberofrecordskey";
    
    private Database          _cacheDB;
    private Database          _provStoreDB;
    private Database          _classDB;
    private Database          _countDB;
    private SecondaryDatabase _idProvStoreDB;
    private Environment       _theEnvironment;
    
    private File _homeDir;
    
    private EntryBinding _iRecordIDBinding;
    private EntryBinding _iRecordBinding;
    private EntryBinding _cacheItemBinding;
    
    public BerkeleyDBJEStorage (String home) {
        try {
            
            File homeDir = new File (home);
            if (!homeDir.exists ()) {
                homeDir.mkdir ();
            }
            _homeDir = homeDir;
            
            EnvironmentConfig envConfig = new EnvironmentConfig ();
            envConfig.setAllowCreate (true);
            envConfig.setTransactional (true);
            //envConfig.setTxnWriteNoSync(buffer);
            _theEnvironment = new Environment (homeDir, envConfig);
            
            
            DatabaseConfig cacheConfig = new DatabaseConfig ();
            cacheConfig.setAllowCreate (true);
            cacheConfig.setSortedDuplicates (true);
            cacheConfig.setTransactional (true);
            Transaction cacheDbTxn = _theEnvironment.beginTransaction (null, null);
            _cacheDB = _theEnvironment.openDatabase (cacheDbTxn, "cache", cacheConfig);
            cacheDbTxn.commit ();
            DatabaseConfig dbConfig = new DatabaseConfig ();
            dbConfig.setAllowCreate (true);
            dbConfig.setTransactional (true);
            
            Transaction psTxn = _theEnvironment.beginTransaction (null, null);
            _provStoreDB = _theEnvironment.openDatabase (psTxn, "provstore", dbConfig);
            psTxn.commit ();
            
            Transaction classTxn = _theEnvironment.beginTransaction (null, null);
            _classDB = _theEnvironment.openDatabase (classTxn, "classDb", dbConfig);
            classTxn.commit ();
            
            Transaction countTxn = _theEnvironment.beginTransaction (null, null);
            _countDB = _theEnvironment.openDatabase (countTxn, "countDb", dbConfig);
            countTxn.commit ();
            
            // Instantiate the class catalog
            StoredClassCatalog classCatalog = new StoredClassCatalog (_classDB);
            
            // Create bindings
            _iRecordIDBinding = new SerialBinding (classCatalog, String.class);
            _iRecordBinding   = new SerialBinding (classCatalog, InteractionRecord.class);
            _cacheItemBinding = new SerialBinding (classCatalog, CacheableItem.class);
            
            IRSecondaryKeyCreator irskc = new IRSecondaryKeyCreator (_iRecordBinding, _iRecordIDBinding);
            SecondaryConfig irSecConfig = new SecondaryConfig ();
            irSecConfig.setKeyCreator (irskc);
            irSecConfig.setAllowCreate (true);
            irSecConfig.setTransactional (true);
            
            //Perform the actual open
            
            
            String secDbName = "provIRSecondaryDB";
            Transaction secDbTxn = _theEnvironment.beginTransaction (null, null);
            _idProvStoreDB = _theEnvironment.openSecondaryDatabase (secDbTxn, secDbName, _provStoreDB, irSecConfig);
            secDbTxn.commit ();
            
            
        } catch (Exception e) {
            e.printStackTrace ();
        }
    }
    
    public void record (GlobalPAssertionKey gpid, Element asserter, Element assertion, String kindOfAssertion) throws Exception {
        
        InteractionKey irid = gpid.getInteractionKey ();
        DatabaseEntry theSecKey = new DatabaseEntry ();
        _iRecordIDBinding.objectToEntry (irid.toString (), theSecKey);
        
        DatabaseEntry theKey  = new DatabaseEntry ();
        DatabaseEntry theData = new DatabaseEntry ();
        boolean noKey = true;
        
        // Get the interaction record and a main key
        Transaction getIrTxn = _theEnvironment.beginTransaction (null, null);
        InteractionRecord ir;
        try {
            if (_idProvStoreDB.get (getIrTxn, theSecKey, theKey, theData, LockMode.DEFAULT) == OperationStatus.SUCCESS)  {
                ir = (InteractionRecord)_iRecordBinding.entryToObject (theData);
                noKey = false;
            } else {
                ir = new InteractionRecord (irid, null, null);
            }
        } catch (DatabaseException dbe) {
            getIrTxn.abort ();
            throw dbe;
        }
        
        getIrTxn.commit ();
        
        // Get the view
        View selectedView = null;
        if (gpid.getViewName ().equalsIgnoreCase (Constants.CLIENT_VIEW)) {
            selectedView = ir.getSenderView ();
            if (selectedView == null) {
                selectedView = new View (new LinkedList (), new LinkedList (), new LinkedList (), -1, asserter);
                ir.setSenderView (selectedView);
            }
        } else if (gpid.getViewName ().equalsIgnoreCase (Constants.SERVICE_VIEW)) {
            selectedView = ir.getReceiverView ();
            if (selectedView == null) {
                selectedView = new View (new LinkedList (), new LinkedList (), new LinkedList (), -1, asserter);
                ir.setReceiverView (selectedView);
            }
        }
        
        // Determine the kind of passertion and add the asseriton to the view in the right spot
        if (kindOfAssertion.equals (MemoryStorage.INTERACTION_PA)) {
            PStructureDeserialiser psd = new PStructureDeserialiser ();
            InteractionPAssertion ipa = psd.getInteractionPAssertion (assertion);
            selectedView.getInteractionPAssertions ().add (ipa);
        } else if (kindOfAssertion.equals (MemoryStorage.ACTOR_STATE_PA)) {
            ActorStatePAssertion apa = new ActorStatePAssertion (gpid.getLocalPAssertionID (), assertion);
            selectedView.getActorStatePAssertions ().add (apa);
        } else if (kindOfAssertion.equals (MemoryStorage.RELATIONSHIP_PA)) {
            PStructureDeserialiser psd = new PStructureDeserialiser ();
            Relationship rel = psd.getRelationshipAssertion (assertion);
            selectedView.getRelationships ().add (rel);
        } else if (kindOfAssertion.equals (MemoryStorage.SUBMISSION_FINISHED)) {
            if (selectedView.getNumberOfExpectedPAssertions () == -1) {
                PStructureDeserialiser psd = new PStructureDeserialiser ();
                int noem = psd.getNumberOfExpectedMsgs (assertion);
                selectedView.setNumberOfExpectedPAssertions (noem);
            }
        }
        
        _iRecordBinding.objectToEntry (ir, theData);
        
        Transaction txn = _theEnvironment.beginTransaction (null, null);
        if (noKey) {
            Long total;
            DatabaseEntry totalKey  = new DatabaseEntry (TOTAL_NUM_RECORDS_KEY.getBytes ("UTF-8"));
            DatabaseEntry totalData = new DatabaseEntry ();
            EntryBinding longBinding = TupleBinding.getPrimitiveBinding (Long.class);
            
            if (_countDB.get (null, totalKey, totalData, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                total = (Long)longBinding.entryToObject (totalData);
                long totalNew = total.longValue ();
                total = new Long (totalNew + 1);
            } else {
                total = new Long (1);
            }
            
            // Update the count
            longBinding.objectToEntry (total, totalData);
            try {
                _countDB.put (txn, totalKey, totalData);
            } catch (DatabaseException dbe) {
                System.out.println ("BDJE: Error updating record count");
                txn.abort ();
                throw dbe;
            }
            
            // Create the key for the database entry that is the key
            // for this interaction record
            Long primary = new Long (total.longValue () - 1);
            longBinding.objectToEntry (primary, theKey);
        }
        
        
        // Put it as normal
        try {
            _provStoreDB.put (txn, theKey, theData);
        } catch (DatabaseException dbe) {
            System.out.println ("BDJE: Could not put new record");
            txn.abort ();
            throw dbe;
        }
        
        txn.commit ();
    }
    
    public InteractionRecord getInteractionRecord (InteractionKey irid) throws Exception {
        DatabaseEntry theKey  = new DatabaseEntry ();
        _iRecordIDBinding.objectToEntry (irid.toString (), theKey);
        DatabaseEntry theData = new DatabaseEntry ();
        Transaction txn = _theEnvironment.beginTransaction (null, null);
        try {
            if (_provStoreDB.get (txn, theKey, theData, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit ();
                return (InteractionRecord)_iRecordBinding.entryToObject (theData);
            } else {
                txn.commit ();
                throw new Exception ("BerkeleyDBJEStorage: No interaction record at that id");
            }
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
        
    }
    
    public InteractionRecord getInteractionRecord (long index) throws Exception {
        
        EntryBinding longBinding = TupleBinding.getPrimitiveBinding (Long.class);
        DatabaseEntry theKey  = new DatabaseEntry ();
        longBinding.objectToEntry (new Long (index), theKey);
        DatabaseEntry theData = new DatabaseEntry ();
        
        Transaction txn = _theEnvironment.beginTransaction (null, null);
        try {
            if (_provStoreDB.get (txn, theKey, theData, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit ();
                return (InteractionRecord)_iRecordBinding.entryToObject (theData);
            } else {
                txn.commit ();
                throw new Exception ("BerkeleyDBJEStorage: No interaction record at that index");
            }
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
    }
    
    public long getNumberOfInteractionRecords () throws Exception {
        DatabaseEntry totalKey  = new DatabaseEntry (TOTAL_NUM_RECORDS_KEY.getBytes ("UTF-8"));
        DatabaseEntry totalData = new DatabaseEntry ();
        EntryBinding longBinding = TupleBinding.getPrimitiveBinding (Long.class);
        Long total;
        
        Transaction txn = _theEnvironment.beginTransaction (null, null);
        try {
            if (_countDB.get (txn, totalKey, totalData, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit ();
                total = (Long)longBinding.entryToObject (totalData);
            } else {
                txn.commit ();
                total = new Long (0);
            }
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
        
        return total.longValue ();
    }
    
    public boolean cacheEnabled () {
        return true;
    }
    
    public void cache (CacheableItem ci) throws Exception {
        String cacheStr = "cache";
        DatabaseEntry theCacheKey = new DatabaseEntry (cacheStr.getBytes ("UTF-8"));
        DatabaseEntry theData = new DatabaseEntry ();
        _cacheItemBinding.objectToEntry (ci, theData);
        
        Transaction txn = null;
        try {
            txn = _theEnvironment.beginTransaction (null, null);
            _cacheDB.put (txn, theCacheKey, theData);
            txn.commit ();
        } catch (DeadlockException deadlock) {
            txn.abort ();
            cache (ci);
        } catch (DatabaseException de) {
            txn.abort ();
            throw de;
        }
    }
    
    public synchronized CacheableItem getCachedItem () throws Exception {
        CacheableItem ci = null;
        
        Cursor cursor = null;
        Transaction txn = null;
        
        try {
            DatabaseEntry foundKey = new DatabaseEntry ();
            DatabaseEntry foundData = new DatabaseEntry ();
            
            txn = _theEnvironment.beginTransaction (null, null);
            cursor = _cacheDB.openCursor (txn, null);
            
            cursor.getNext (foundKey, foundData, LockMode.DEFAULT);
            
            cursor.delete ();
            cursor.close ();
            txn.commit ();
            
            ci = (CacheableItem) _cacheItemBinding.entryToObject (foundData);
        } catch (NullPointerException npe) {
            
            //left intentionally blank
        } catch (Exception e) {
            cursor.close ();
            txn.abort ();
        }
        return ci;
    }
    
    public void shutdown () throws Exception {
        _theEnvironment.sync ();
        
        _idProvStoreDB.close ();
        _cacheDB.close ();
        _provStoreDB.close ();
        _classDB.close ();
        _countDB.close ();
        _theEnvironment.close ();
    }
    
    public void delete () throws Exception {
        shutdown ();
        _homeDir.delete ();
    }
    
    
}
