package org.pasoa.storage.memory;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.NoSuchElementException;

import org.pasoa.common.CacheableItem;
import org.pasoa.common.Constants;
import org.pasoa.common.structure.ActorStatePAssertion;
import org.pasoa.common.structure.GlobalPAssertionKey;
import org.pasoa.common.structure.InteractionPAssertion;
import org.pasoa.common.structure.InteractionRecord;
import org.pasoa.common.structure.InteractionKey;
import org.pasoa.common.structure.Relationship;
import org.pasoa.common.structure.View;
import org.pasoa.common.structure.deserialiser.PStructureDeserialiser;
import org.pasoa.storage.BackendStore;

import org.w3c.dom.Element;


public class MemoryStorage implements BackendStore {
    private LinkedList    _cache;
    private LinkedHashMap _iRecords;
    
    public MemoryStorage () {
        _cache    = new LinkedList ();
        
        // Key is InteractionRecordId
        // Value is InteractionRecord
        _iRecords = new LinkedHashMap ();
    }
    
    public void record (GlobalPAssertionKey gpid, Element asserter, Element assertion, String kindOfAssertion) throws Exception {
        
        InteractionKey irid = gpid.getInteractionKey ();
        System.out.println ("IDID-MS " + irid.toString ());
        // Get the interaction record
        InteractionRecord ir;
        if (_iRecords.containsKey (irid)) {
            System.out.println ("irec contains key");
            ir = (InteractionRecord) _iRecords.get (irid);
        } else {
            System.out.println ("Making new interaction record");
            ir = new InteractionRecord (irid, null, null);
        }
        
        // Get the view
        View selectedView = null;
        if (gpid.getViewName ().equalsIgnoreCase (Constants.CLIENT_VIEW)) {
            selectedView = ir.getSenderView ();
            if (selectedView == null) {
                selectedView = new View (new LinkedList (), new LinkedList (), new LinkedList (), -1, asserter);
                ir.setSenderView (selectedView);
            }
        } else if (gpid.getViewName ().equalsIgnoreCase (Constants.SERVICE_VIEW)) {
            selectedView = ir.getReceiverView ();
            if (selectedView == null) {
                selectedView = new View (new LinkedList (), new LinkedList (), new LinkedList (), -1, asserter);
                ir.setSenderView (selectedView);
            }
        }
        
        // Determine the kind of passertion and add the asseriton to the view in the right spot
        if (kindOfAssertion.equals (MemoryStorage.INTERACTION_PA)) {
            PStructureDeserialiser psd = new PStructureDeserialiser ();
            InteractionPAssertion ipa = psd.getInteractionPAssertion (assertion);
            selectedView.getInteractionPAssertions ().add (ipa);
        } else if (kindOfAssertion.equals (MemoryStorage.ACTOR_STATE_PA)) {
            ActorStatePAssertion apa = new ActorStatePAssertion (gpid.getLocalPAssertionID (), assertion);
            selectedView.getActorStatePAssertions ().add (apa);
        } else if (kindOfAssertion.equals (MemoryStorage.RELATIONSHIP_PA)) {
            PStructureDeserialiser psd = new PStructureDeserialiser ();
            Relationship rel = psd.getRelationshipAssertion (assertion);
            selectedView.getRelationships ().add (rel);
        } else if (kindOfAssertion.equals (MemoryStorage.SUBMISSION_FINISHED)) {
            if (selectedView.getNumberOfExpectedPAssertions () == -1) {
                PStructureDeserialiser psd = new PStructureDeserialiser ();
                int noem = psd.getNumberOfExpectedMsgs (assertion);
                selectedView.setNumberOfExpectedPAssertions (noem);
            }
        }
        
        // Put the InteractionRecord into the hash of interaction records
        _iRecords.put (irid, ir);
        System.out.println ("IRecords size " + _iRecords.size ());
    }
    
    
    public boolean cacheEnabled () {
        return true;
    }
    
    public void cache (CacheableItem ci) throws Exception {
        _cache.add (ci);
        
    }
    
    public CacheableItem getCachedItem () throws Exception {
        CacheableItem ci = null;
        
        try {
            ci = (CacheableItem) _cache.getFirst ();
            _cache.removeFirst ();
        } catch (NoSuchElementException nsee) {
            // intentionally left blank so null will be returned
        }
        
        return ci;
    }
    
    public InteractionRecord getInteractionRecord (long index) throws Exception {
        InteractionRecord ir = null;
        Iterator records = _iRecords.values ().iterator ();
        int position = 0;
        
        while (records.hasNext ()) {
            if (position == index) {
                ir = (InteractionRecord) records.next ();
                break;
            } else {
                records.next ();
            }
            position++;
        }
        return ir;
    }
    
    public long getNumberOfInteractionRecords () throws Exception {
        return _iRecords.size ();
    }
    
    public void shutdown () throws Exception {
        // memory does not need to shutdown
    }
    
    public void delete () throws Exception {
        
    }
    
    
    
}
