package org.pasoa.util.httpsoap;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.pasoa.common.Constants;
import org.pasoa.common.structure.ActorStatePAssertion;
import org.pasoa.common.structure.InteractionPAssertion;
import org.pasoa.common.structure.ObjectID;
import org.pasoa.common.structure.Record;
import org.pasoa.common.structure.RecordSf;
import org.pasoa.common.structure.Relationship;
import org.pasoa.dommap.DOMSerialiser;

public class RecordRequestDocument extends SOAPDocument {
    private static String pRecordPrefix = "pr";
    private static String pStructPrefix = "ps";
    
    public RecordRequestDocument (Record request) {
        super (recordSpaces ());
        
        DOMSerialiser builder = new DOMSerialiser (body);
        
        builder.namespace (Constants.PRECORD_NS);
        builder.start     ("record");
        builder.start     ("identifiedContent");
        builder.namespace (Constants.PSTRUCT_NS);
        builder.start     ("interactionKey");
        builder.single    ("messageSource", request.getIdentifiers ().getMessageSource ());
        builder.single    ("messageSink",   request.getIdentifiers ().getMessageSink ());
        builder.single    ("interactionId", request.getIdentifiers ().getInteractionID ());
        builder.end       ();
        builder.single    ("viewKind",      request.getViewKind ());
        builder.single    ("asserter",      request.getAsserter ());
        builder.namespace (Constants.PRECORD_NS);
        builder.start     ("content");
        builder.namespace (Constants.PSTRUCT_NS);
        
        if (request.getPAssertion () instanceof InteractionPAssertion) {
            InteractionPAssertion ipa = (InteractionPAssertion) request.getPAssertion ();
            builder.start  ("interactionPAssertion");
            builder.single ("localPAssertionId",  ipa.getLocalPAssertionID ());
            builder.single ("documentationStyle", ipa.getDocumentationStyle ());
            builder.append (ipa.getMessageBody ());
        }
        if (request.getPAssertion () instanceof ActorStatePAssertion) {
            ActorStatePAssertion aspa = (ActorStatePAssertion) request.getPAssertion ();
            builder.start  ("actorStatePAssertion");
            builder.single ("localPAssertionId",  aspa.getLocalPAssertionID ());
            builder.append (aspa.getActorStateBody ());
        }
        if (request.getPAssertion () instanceof Relationship) {
            Relationship rpa     = (Relationship) request.getPAssertion ();
            Iterator     eachObj = rpa.getObjectIDs ().iterator ();
            ObjectID     object;
            
            builder.start  ("relationshipPAssertion");
            builder.single ("localPAssertionId",  rpa.getLocalPAssertionID ());
            builder.start  ("subjectId");
            builder.single ("localPAssertionId",  rpa.getSubject ().getLocalPAssertionID ());
            builder.single ("dataAccessor",       rpa.getSubject ().getDataAccessor ());
            builder.single ("parameterName",      rpa.getSubject ().getParameterName ());
            builder.end    ();
            builder.single ("relation",           rpa.getRelation ());
            while (eachObj.hasNext ()) {
                object = (ObjectID) eachObj.next ();
                builder.start  ("objectId");
                builder.single ("messageSource",     object.getGlobalPAssertionKey ().getInteractionKey ().getMessageSource ());
                builder.single ("messageSink",       object.getGlobalPAssertionKey ().getInteractionKey ().getMessageSink ());
                builder.single ("interactionId",     object.getGlobalPAssertionKey ().getInteractionKey ().getInteractionID ());
                builder.single ("viewKind",          object.getGlobalPAssertionKey ().getViewName ());
                builder.single ("localPAssertionId", object.getGlobalPAssertionKey ().getLocalPAssertionID ());
                builder.single ("dataAccessor",      object.getObjectDataAccessor ());
                builder.single ("parameterName",     object.getParameterName ());
                builder.end    ();
            }
        }
        if (request instanceof RecordSf) {
            builder.namespace (Constants.PRECORD_NS);
            builder.single    ("submissionFinished", ((RecordSf) request).getNumberOfExpectedPAssertions ());
        }
    }
    
    private static Map recordSpaces () {
        Map spaces = new HashMap ();
        spaces.put (pRecordPrefix, Constants.PRECORD_NS);
        spaces.put (pStructPrefix, Constants.PSTRUCT_NS);
        return spaces;
    }
}
