/*******************************************************************************
 * Copyright (c) 2004 University of Southampton
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE.
 *******************************************************************************/
package org.pasoa.wrappers.axishandler;


import org.pasoa.util.ClientLib;
import org.pasoa.common.Constants;
import org.pasoa.common.structure.*;
import org.pasoa.common.structure.deserialiser.PStructureDeserialiser;
import org.pasoa.dommap.DOMWriter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.apache.axis.handlers.BasicHandler;
import org.apache.axis.utils.XMLUtils;
import org.apache.axis.MessageContext;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.Node;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPConstants;
import javax.xml.soap.Name;
import org.apache.axis.message.SOAPHeaderElement;

import java.io.IOException;
import java.io.StringReader;
import java.net.InetAddress;

import org.apache.axis.components.uuid.SimpleUUIDGen;

import java.net.URL;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

/**
 * An axis handler that automatically sends all
 * request/responses to a provenance service
 * 
 * @author Paul Groth pg03r@ecs.soton.ac.uk
 */
public class ProvenanceHandler extends BasicHandler
{

    public static final String P_RECORD_SCHEMA = Constants.PRECORD_NS;
    public static final String P_XQUERY_SCHEMA = Constants.WSXQUERY_NS;
    public static final String PHEADER_NS      = Constants.PHEADER_NS;
    public static final String PSTRUCT_NS      = Constants.PSTRUCT_NS;
    
    public void invoke(MessageContext msgContext)
    {
        try {

            URL provServ;
            if (ProvenanceHandler.getProvenanceService () == null) {
                String prs = (String) getOption ("DefaultProvenanceService");
                if (prs != null) {
                    provServ = new URL (prs);
                    ProvenanceHandler.setProvenanceService (provServ);
                }
                else {
                    System.out.println ("ProvenanceHandler: No Default Provenance Service. Not Engaging");
                    return;
                }

            }
            else {
                provServ = ProvenanceHandler.getProvenanceService ();
            }

            
            System.out.println ("ProvenanceHandler: New Call " + _gotCalled);
            

            SimpleUUIDGen uuidGen = new SimpleUUIDGen ();
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
            DocumentBuilder        builder = factory.newDocumentBuilder ();
            
            
            // Get the soap message
            SOAPMessage soapMsg = msgContext.getCurrentMessage();
            SOAPBody soapBody = soapMsg.getSOAPPart ().getEnvelope ().getBody ();
            System.out.println ("Before Processsing ....");
            //System.out.println (XMLUtils.ElementToString(soapMsg.getSOAPPart().getEnvelope()));

            // Ignore URLS
            Vector ignoreList = new Vector ();
            ignoreList.addElement (P_RECORD_SCHEMA);
            ignoreList.addElement (P_XQUERY_SCHEMA);
           
            // Determine if it is a message to a provenance service
            Iterator children = soapBody.getChildElements ();
            while (children.hasNext ()) {
                String uri = ((SOAPElement) children.next ()).getElementName ().getURI ();
                //System.out.println ("RedirectHandler: " + uri);
                if (ignoreList.contains (uri)) {
                    System.out.println ("ProvenanceHandler: Ignoring not contacting provenance service");
                    return;
                }
            }

           
            
            String actor = null;
            InteractionKey irid = null;
            boolean removeHeader = false;
            
            // Check to see if there is a pheader
            Iterator headerElems = soapMsg.getSOAPPart ().getEnvelope ().getHeader ().examineHeaderElements (SOAPConstants.URI_SOAP_ACTOR_NEXT);
            SOAPHeaderElement pHeader = null;
            while (headerElems.hasNext ()) {
                //System.out.println ("ProvenanceHandler: Printing found SOAP header");
               
                SOAPHeaderElement she = (SOAPHeaderElement) headerElems.next ();
                //System.out.println ();
                //System.out.println(she);
                Iterator nsPrefixes = she.getNamespacePrefixes();
                while (nsPrefixes.hasNext()) {
                    if (she.getNamespaceURI((String)nsPrefixes.next()).equals(PHEADER_NS)) {
                        pHeader = she;
                        break;
                    }
                }
            }
            
            // If there is a pheader then this must be the service side (sender)
            if (pHeader != null) {
                //System.out.println ("Service Side");
                PStructureDeserialiser psd = new PStructureDeserialiser();
               
                irid = psd.getInteractionRecordID(pHeader.getAsDocument().getDocumentElement());
                
                //System.out.println (irid.getClientID());
                //System.out.println(irid.getServiceID());
                //System.out.println(irid.getInteractionID());
                
                actor = Constants.SERVICE_VIEW;
                
                NodeList tracers = pHeader.getAsDocument().getDocumentElement().getElementsByTagNameNS(PHEADER_NS, "tracer");
                for (int t = 0; t <  tracers.getLength(); t++) {
                    
                    String tValue = tracers.item(t).getTextContent();
                    //System.out.println ("Found a tracer " + tValue );
                    ProvenanceHandler.addATracer (tValue);
                }
                removeHeader = true;
                //System.out.println ("Detaching node");
                soapMsg.getSOAPHeader().removeChild(pHeader);
                
                Name pheader = soapMsg.getSOAPPart ().getEnvelope().createName("pheader", "ph", PHEADER_NS);
                
                javax.xml.soap.SOAPHeaderElement she = soapMsg.getSOAPHeader().addHeaderElement(pheader);
                 SOAPElement ids = she.addChildElement("interactionKey", "ps", PSTRUCT_NS);
                
                ids.addChildElement("messageSource", "ps", PSTRUCT_NS).addChildElement("EndpointReference", "", "").addChildElement("Address").addTextNode(irid.getMessageSource().getTextContent());
                ids.addChildElement("messageSink", "ps", PSTRUCT_NS).addChildElement("EndpointReference", "", "").addChildElement("Address").addTextNode(irid.getMessageSink().getTextContent());
                
                SOAPElement iidSe = ids.addChildElement("interactionId", "ps", PSTRUCT_NS);
                iidSe.addTextNode(irid.getInteractionID());
                
                
                Set tracerSet = ProvenanceHandler.getTracers();
                if (tracerSet != null) {
                   Iterator iterTracers = tracerSet.iterator();
                    while (iterTracers.hasNext()) {
                        String tra = (String)iterTracers.next();
                        SOAPElement imd = she.addChildElement("interactionMetaData", "ph", PHEADER_NS);
                        imd.addChildElement("tracer").addTextNode(tra);
                    }
                }
                
                
                
                
                pHeader = (SOAPHeaderElement)she;
                
                
            }
            
            // Otherwise, if there is no pheader then this must be the client (receiver)
            // so a pheader must be added
            else {
                //System.out.println ("ProvenanceHandler: Client Invoke");
                actor = Constants.CLIENT_VIEW;
               
                String clientIdStr = "http://" + InetAddress.getLocalHost().getCanonicalHostName(); 
                String clientID = "<EndpointReference xmlns = \"\"><Address>" + clientIdStr + "</Address></EndpointReference>";
                
                URL servURL = new URL ((String) msgContext.getProperty (MessageContext.TRANS_URL));
                
                String serviceID = "<EndpointReference xmlns = \"\"><Address>" + servURL.toExternalForm() + "</Address></EndpointReference>";
                
                String interactionID = "urn:uuid:" + uuidGen.nextUUID();
        
                Element clientIDElem = builder.parse (new InputSource (new StringReader (clientID))).getDocumentElement ();
                Element serviceIDElem = builder.parse (new InputSource (new StringReader (serviceID))).getDocumentElement ();
                
                irid = new InteractionKey (clientIDElem, serviceIDElem, interactionID);
                
                Name pheader = soapMsg.getSOAPPart ().getEnvelope().createName("pheader", "ph", PHEADER_NS);
                
                javax.xml.soap.SOAPHeaderElement she = soapMsg.getSOAPHeader().addHeaderElement(pheader);
                 SOAPElement ids = she.addChildElement("interactionKey", "ps", PSTRUCT_NS);
                
                ids.addChildElement("messageSource", "ps", PSTRUCT_NS).addChildElement("EndpointReference", "", "").addChildElement("Address").addTextNode(irid.getMessageSource().getTextContent());
                ids.addChildElement("messageSink", "ps", PSTRUCT_NS).addChildElement("EndpointReference", "", "").addChildElement("Address").addTextNode(irid.getMessageSink().getTextContent());
                
                SOAPElement iidSe = ids.addChildElement("interactionId", "ps", PSTRUCT_NS);
                iidSe.addTextNode(irid.getInteractionID());
                
                
                
                Set tracerSet = ProvenanceHandler.getTracers();
                if (tracerSet != null) {
                   Iterator iterTracers = tracerSet.iterator();
                    while (iterTracers.hasNext()) {
                        String tra = (String)iterTracers.next();
                        SOAPElement imd = she.addChildElement("interactionMetaData", "ph", PHEADER_NS);
                        imd.addChildElement("tracer").addTextNode(tra);
                    }
                }
                SOAPElement itemElem = she.addChildElement("interactionMetaData", "ph", PHEADER_NS);
                itemElem.addChildElement("tracer").addTextNode("urn:uuid:" + uuidGen.nextUUID());
               
            }

            //System.out.println ("---------------XML UTILS-----------------");
            //System.out.println (XMLUtils.ElementToString(soapMsg.getSOAPPart().getEnvelope()));
            //System.out.println ("---------------DOMWriter-----------------");
            //System.out.println (DOMWriter.writeToString(soapMsg.getSOAPPart().getEnvelope()));

            if (irid != null && actor != null) {
                StringBuffer content = new StringBuffer();
                content.append("<ps:interactionPAssertion>");
                content.append("<ps:localPAssertionId>");
                content.append("urn:uuid: " + uuidGen.nextUUID());
                content.append("</ps:localPAssertionId>");
                content.append("<ps:documentationStyle>");
                content.append("http://www.pasoa.org/ontologies/transformation/none");
                content.append("</ps:documentationStyle>");
                content.append(XMLUtils.ElementToString(soapMsg.getSOAPPart().getEnvelope()));
                content.append("</ps:interactionPAssertion>");
                
                String asserterStr = "<name>http://" + InetAddress.getLocalHost().getCanonicalHostName() + "</name>"; 
               
                Element asserter = builder.parse (new InputSource (new StringReader (asserterStr))).getDocumentElement ();
                
                ClientLib cl = new ClientLib();
                String recDoc = cl.makeRecordDocument(irid, actor, asserter, content.toString());
                try {
                    cl.record(recDoc, provServ);
                } 
                catch (SAXException sae) {
                    System.out.println ("ProvenanceHandler: Unable to parse response from provenance store.");
                    System.out.println ("ProvenanceHandler: Perhaps the URL to the store is incorrect?");
                }
                catch (IOException ioe){
                    System.out.println ("ProvenanceHandler: Unable to send assertion to the provenance store");
                    System.out.println ("ProvenanceHandler: Is there a problem with the network?");
                }
            }
            
           if (removeHeader) {
                //System.out.println ("Removing header");
                soapMsg.getSOAPHeader().removeChild(pHeader); 
            }

            System.out.println ("RedirectHandler: Finished for call " + _gotCalled);
            _gotCalled++;

        }
        
        catch (Exception e) {
            e.printStackTrace ();
        }

    }
    
   
    public static void addIdentifiersToElement (InteractionKey irid, Element elem) throws Exception {
         
       System.out.println("what");
         //elem.appendChild(elem.getOwnerDocument().importNode(irid.getClientID(), true));
         //elem.appendChild(elem.getOwnerDocument().importNode(irid.getMessageSource(), true));
         //elem.appendChild(elem.getOwnerDocument().importNode(irid.getServiceID(), true));
         //elem.appendChild(elem.getOwnerDocument().importNode(irid.getMessageSink(), true));
      
         //Element iid = elem.getOwnerDocument().createElement("ps:interactionId");
         //iid.setTextContent(irid.getInteractionID());
         //elem.appendChild(iid);
         
    }
    public static String interactionRecordIDToIdentifiersString (InteractionKey irid)
    {
        StringBuffer sb = new StringBuffer();
        
        sb.append("<ps:interactionKey>");
        sb.append("<ps:messageSource>");
        sb.append(DOMWriter.writeToString(irid.getMessageSource()));
        sb.append("</ps:messageSource>");
        sb.append("<ps:messageSink>");
        sb.append(DOMWriter.writeToString(irid.getMessageSink()));
        sb.append("</ps:messageSink>");
        sb.append("<ps:interactionId>");
        sb.append(irid.getInteractionID());
        sb.append("</ps:interactionId>");
        sb.append("</ps:interactionKey>");
        
        
        return sb.toString();
    }

    /**
     * Overides the default provenance service for this thread of execution.
     * 
     * @param provServ  the url of the provenance service
     */
    public static void setProvenanceService(URL provServ)
    {
        _provService.set (provServ);
    }

    /**
     * Get the url of the provenance service. If a service has not yet been
     * invoked this will return null. After invocation this will return the last
     * provenance service used in this thread.
     * 
     * @return  the URL of the provenance service in use
     */
    public static URL getProvenanceService()
    {
        return (URL) _provService.get ();
    }

    
    public static void addATracer(String sid)
    {
        Set tracers = (Set)_tracerSet.get();
        if (tracers == null) {
           tracers = new HashSet();
        }
        tracers.add(sid);
        
        _tracerSet.set (tracers);
    }

    /**
     * Get the tracers for this thread. If a service has not yet been invoked
     * this will return null, unless specifically set. The handler will generate
     * a tracer automatically for every thread.
     * 
     * @return   the session id in use for this thread
     */
    public static Set getTracers()
    {
        return (Set) _tracerSet.get ();
    }

    // This variable stores a string session id for a particular thread
    private static ThreadLocal _tracerSet = new ThreadLocal ();
    private static ThreadLocal _provService = new ThreadLocal ();

   
    private int _gotCalled = 1;
}